SystemOrganization addCategory: #'Parsing-Derivatives-Test'!
SystemOrganization addCategory: #'Parsing-Derivatives'!
SystemOrganization addCategory: #'Parsing-Derivatives-Support'!
SystemOrganization addCategory: #'Parsing-Derivatives-Support-Tests'!

!SequenceableCollection methodsFor: '*Parsing-Derivatives' stamp: 'fbs 4/15/2013 21:02'!
asSequenceParser
	self ifEmpty: [^ DerivingParser empty].
	^ self allButFirst
		inject: self first asParser
		into: [:acc :each | acc then: each asParser].! !

Error subclass: #ArityMismatch
	instanceVariableNames: 'expectedArgCount actualArgCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!ArityMismatch commentStamp: 'fbs 7/20/2012 22:01' prior: 0!
I represent someone trying to invoke a block with the incorrect number of arguments.!

!ArityMismatch class methodsFor: 'as yet unclassified' stamp: 'fbs 7/20/2012 22:03'!
actual: actual expected: expected
	^ self new initializeWithActualCount: actual expectedCount: expected.! !

!ArityMismatch methodsFor: 'accessing' stamp: 'fbs 7/20/2012 22:02'!
actualArgCount
	^ actualArgCount.! !

!ArityMismatch methodsFor: 'accessing' stamp: 'fbs 7/20/2012 22:02'!
expectedArgCount
	^ expectedArgCount.! !

!ArityMismatch methodsFor: 'initialize-release' stamp: 'fbs 7/20/2012 22:02'!
initializeWithActualCount: actual expectedCount: expected
	actualArgCount := actual.
	expectedArgCount := expected.! !

Error subclass: #ParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!ParseError class methodsFor: 'as yet unclassified' stamp: 'fbs 3/22/2013 08:53'!
signalLine: lineNumber column: columnNumber error: aString.
	^ self signal: ('Error at line {1}, column {2}: {3}.'
		format: {lineNumber asString. columnNumber asString. aString}).! !

Error subclass: #UnresolvedDelegate
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!UnresolvedDelegate commentStamp: 'fbs 10/10/2012 19:06' prior: 0!
I signal that you've tried to access the parser to which a DelegateParser points, only you haven't told the parser which parser that is.!

!UnresolvedDelegate class methodsFor: 'instance creation' stamp: 'fbs 10/10/2012 19:08'!
accessing: aDelegateParser
	^ (self new parser: aDelegateParser) signal.! !

!UnresolvedDelegate methodsFor: 'accessing' stamp: 'fbs 10/10/2012 19:08'!
parser
	^ parser.! !

!UnresolvedDelegate methodsFor: 'accessing' stamp: 'fbs 10/10/2012 19:08'!
parser: aDelegateParser
	parser := aDelegateParser.! !

TestCase subclass: #DelayedStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!DelayedStreamTest methodsFor: 'testing' stamp: 'fbs 10/30/2012 11:34'!
testCanBeInstantiated
	self shouldnt: [DelayedStream on: [#(1)]] raise: Error.! !

!DelayedStreamTest methodsFor: 'testing' stamp: 'fbs 10/31/2012 08:14'!
testCanWrapAnotherStream
	| delay |
	delay := DelayedStream on: [#(1) reading].
	self assert: #(1) equals: (delay read: 1).
	self should: [(DelayedStream on: [#(1) reading]) read: 2] raise: Incomplete.! !

!DelayedStreamTest methodsFor: 'testing' stamp: 'fbs 10/30/2012 12:29'!
testContentsSpeciesDependsOnBlockReturnType
	self assert: Array equals: (DelayedStream on: [#()]) contentsSpecies.
	self assert: ByteArray equals: (DelayedStream on: [#[]]) contentsSpecies.! !

!DelayedStreamTest methodsFor: 'testing' stamp: 'fbs 10/30/2012 13:38'!
testUnderlyingStreamCreatedLazily
	| delay hasRun |
	hasRun := false.
	delay := DelayedStream on: [hasRun := true. #(1)].
	self deny: hasRun.
	self assert: #(1) equals: (delay read: 1).
	self assert: hasRun.! !

TestCase subclass: #DerivingParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!DerivingParserTest methodsFor: 'testing' stamp: 'fbs 10/24/2012 21:34'!
testDeriverBlockReturnsNewBlock
	self deny: DerivingParser deriverBlock == DerivingParser deriverBlock.! !

!DerivingParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 4/15/2013 22:06'!
testEmptyIsEmpty
	| e |
	e := DerivingParser isEmptyBlock.
	self assert: (e value: DerivingParser empty).! !

!DerivingParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 2/15/2013 09:29'!
testEmptyIsEmptyParser
	self assert: DerivingParser empty isEmptyParser.! !

!DerivingParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 9/4/2012 08:55'!
testEmptyIsMemoized
	self assert: DerivingParser empty == DerivingParser empty.! !

!DerivingParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 9/4/2012 08:55'!
testEmptyStringIsMemoized
	self assert: DerivingParser emptyString == DerivingParser emptyString.! !

!DerivingParserTest methodsFor: 'testing - testing' stamp: 'fbs 10/11/2012 09:41'!
testInADelegateCycleAllParsersAreChildren
	| p d |
	d := DelegateParser new.
	p := DerivingParser emptyString or: ($a asParser then: d).
	d parser: p.
	self assert: (p isChildOf: d).
	self assert: (d isChildOf: p).! !

!DerivingParserTest methodsFor: 'testing' stamp: 'fbs 10/24/2012 21:33'!
testIsEmptyBlockReturnsNewBlock
	self deny: DerivingParser isEmptyBlock == DerivingParser isEmptyBlock.! !

!DerivingParserTest methodsFor: 'testing' stamp: 'fbs 10/24/2012 21:34'!
testParseNullBlockReturnsNewBlock
	self deny: DerivingParser parseNullBlock == DerivingParser parseNullBlock.! !

!DerivingParserTest methodsFor: 'testing' stamp: 'fbs 10/24/2012 09:47'!
testUsesSpecialInspector
	self assert: ParserInspector equals: DerivingParser empty inspectorClass! !

TestCase subclass: #DerivingParserTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

DerivingParserTestCase subclass: #CatTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!CatTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:34'!
sampleParser
	^ $a asParser then: $b asParser.! !

!CatTest methodsFor: 'testing - parsing' stamp: 'fbs 2/8/2013 08:42'!
testCompactionCompactsSubparsers
	| c p l r |
	l := $a asParser.
	r := $b asParser.
	p := (DelegateParser new parser: l) then: (DelegateParser new parser: r).
	c := ParserCompacter value: p.
	self assert: c isCat.
	self assert: l equals: c first.
	self assert: r equals: c second.! !

!CatTest methodsFor: 'testing - parsing' stamp: 'fbs 2/8/2013 08:46'!
testCompactionOfCompactParsersIsItself
	| p |
	p := $a asParser then: $b asParser.
	self assert: p == (ParserCompacter value: p).! !

!CatTest methodsFor: 'testing - parsing' stamp: 'fbs 2/8/2013 08:46'!
testCompactionWithSingleElementFirstParseTreeIsRed
	| c p |
	p := (EpsStar treeSet: {$a}) then: $b asParser.
	c := ParserCompacter value: p.
	self assert: c isRed.! !

!CatTest methodsFor: 'testing - parsing' stamp: 'fbs 2/8/2013 08:46'!
testCompactionWithSingleElementSecondParseTreeIsRed
	| c p |
	p := $a asParser then: (EpsStar treeSet: {$b}).
	c := ParserCompacter value: p.
	self assert: c isRed.! !

!CatTest methodsFor: 'testing - deriving' stamp: 'fbs 4/15/2013 21:22'!
testDerivativeFirstParserNullable
	| d e p r |
	e := DerivingParser isEmptyBlock.
	r := 1 asParser.
	p := (EpsStar treeSet: #(1)) then: r.
	d := DerivingParser deriverBlock value: 1 value: p.
	self assert: d isUnion.
	self assert: d left isCat.
	self assert: d left first isEpsStar.
	self assert: #(1) asSet equals: d left second treeSet.
	self assert: d right isCat.
	self assert: (e value: d right first).
	self assert: r equals: d right second.! !

!CatTest methodsFor: 'testing - deriving' stamp: 'fbs 10/16/2012 17:08'!
testDerivativeFollowsLeibniz
	"(ab)' = a'b + ab'."
	| p d |
	p := 1 asParser then: 2 asParser.
	d := DerivingParser deriverBlock value: 1 value: p.
	self assert: d isCat.
	self assert: d first isEpsStar.
	self assert: #(1) asSet equals: d first treeSet.
	self assert: d second isLiteral.
	self assert: 2 equals: d second token.! !

!CatTest methodsFor: 'testing - accessing' stamp: 'fbs 8/30/2012 22:17'!
testFirst
	| p p1 p2 |
	p1 := $a asParser.
	p2 := $b asParser.
	p := p1 then: p2.
	self assert: p1 equals: p first.! !

!CatTest methodsFor: 'testing - generating' stamp: 'fbs 10/26/2012 09:48'!
testGenerateProducesCartesianProduct
	| p |
	p := ($a asParser or: $b asParser) then: ($c asParser or: $d asParser).
	self assert: #(($a $c) ($a $d) ($b $c) ($b $d)) equals: (p generate read: 4).
	self should: [p generate read: 5] raise: Incomplete.! !

!CatTest methodsFor: 'testing - generating' stamp: 'fbs 10/26/2012 18:50'!
testGenerateProducesConcatenationsOfParses
	| p |
	p := $a asParser then: $b asParser.
	self assert: #(($a $b)) equals: (p generate read: 1).
	self should: [p generate read: 2] raise: Incomplete.! !

!CatTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:37'!
testIsCat
	self assert: self sampleParser isCat.! !

!CatTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:43'!
testPrintAsDigraphPrintsSubparsers
	| f s digraph |
	f := $1 asParser.
	s := DerivingParser emptyString.
	digraph := (f then: s) printAsDigraph.
	self assert: (digraph includesSubString: f dotLabel) description: 'digraph doesn''t mention first subparser'.
	self assert: (digraph includesSubString: s dotLabel) description: 'digraph doesn''t mention second subparser'.! !

!CatTest methodsFor: 'testing - accessing' stamp: 'fbs 8/30/2012 22:19'!
testSecond
	| p p1 p2 |
	p1 := $a asParser.
	p2 := $b asParser.
	p := p1 then: p2.
	self assert: p2 equals: p second.! !

!CatTest methodsFor: 'testing - testing' stamp: 'fbs 10/11/2012 09:31'!
testSubParserIsChildOfCat
	| p |
	p := self sampleParser.
	self assert: (p first isChildOf: p).
	self assert: (p second isChildOf: p).! !

!CatTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:28'!
testSubParsersReturnsFirstAndSecond
	| p |
	p := $a asParser then: $b asParser.
	self assert: {p first. p second} equals: p subParsers.! !

!CatTest methodsFor: 'testing - testing' stamp: 'fbs 10/11/2012 09:36'!
testSubSubParserIsChildOfCat
	| c p |
	c := $a asParser.
	p := (c then: $b asParser) then: $c asParser.
	self assert: (c isChildOf: p).! !

!CatTest methodsFor: 'testing - instance creation' stamp: 'fbs 8/30/2012 16:34'!
testWithWithCreatesParser
	Cat with: $a asParser with: $b asParser.! !

DerivingParserTestCase subclass: #DelayedParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!DelayedParserTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 21:28'!
sampleParser
	^ [1 asParser] asParser.! !

!DelayedParserTest methodsFor: 'testing - instance creation' stamp: 'fbs 9/5/2012 09:38'!
testDelayingADelayedParserIsaNoOp
	| p d |
	p := [1 asParser] asParser.
	d := p delay.
	self assert: p == d.! !

!DelayedParserTest methodsFor: 'testing - deriving' stamp: 'fbs 4/15/2013 22:05'!
testDerivativeIsDerivativeOfSubparser
	| e |
	e := DerivingParser isEmptyBlock.
	self assert: (e value: (DerivingParser deriverBlock value: $b value: ($a asParser delay))).! !

!DelayedParserTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:35'!
testDotLabelDescribesParser
	| p |
	p := self sampleParser.
	self assert: (p dotLabel includesSubString: 'Delay').! !

!DelayedParserTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 21:30'!
testForce
	| p sub |
	sub := $a asParser.
	p := sub delay.
	self assert: p force == sub.! !

!DelayedParserTest methodsFor: 'testing - generating' stamp: 'fbs 10/26/2012 19:00'!
testGenerateReturnsSubparsersParses
	| d p |
	p := $a asParser.
	d := p delay.
	self assert: (p generate read: 1) equals: (d generate read: 1).
	self should: [d generate read: 2] raise: Incomplete.! !

!DelayedParserTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 21:29'!
testIsDelayed
	self assert: self sampleParser isDelayed.! !

!DelayedParserTest methodsFor: 'testing - parsing' stamp: 'fbs 10/24/2012 21:41'!
testIsNullable
	"A delay is as nullable as its subparser."
	| n |
	n := DerivingParser isNullableBlock.
	self assert: (n value: (EpsStar treeSet: {1}) delay).
	self deny: (n value: DerivingParser empty delay).! !

!DelayedParserTest methodsFor: 'testing - parsing' stamp: 'fbs 10/12/2012 18:51'!
testParseNullIsThatOfSubparser
	| parseNull |
	parseNull := DerivingParser parseNullBlock.
	self assert: #(1 2) asSet equals: (parseNull value: (EpsStar treeSet: #(1 2)) delay).! !

!DelayedParserTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:44'!
testPrintAsDigraphPrintsSubparsers
	| r digraph |
	r := $1 asParser.
	digraph := r delay printAsDigraph.
	self assert: (digraph includesSubString: r dotLabel) description: 'digraph doesn''t mention subparser'.! !

!DelayedParserTest methodsFor: 'testing - testing' stamp: 'fbs 10/11/2012 09:32'!
testSubParserIsChildOfSelf
	| p |
	p := self sampleParser.
	self assert: (p parser isChildOf: p).! !

!DelayedParserTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:29'!
testSubParsersReturnsForced
	| p |
	p := $a asParser delay.
	self assert: {p parser} equals: p subParsers.! !

DerivingParserTestCase subclass: #DelegateParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!DelegateParserTest methodsFor: 'fixtures' stamp: 'fbs 8/31/2012 12:10'!
sampleParser
	^ DelegateParser new parser: $a asParser.! !

!DelegateParserTest methodsFor: 'testing - parsing' stamp: 'fbs 4/15/2013 22:06'!
testCompactionIsCompactionOfSubparser
	| d e p |
	e := DerivingParser isEmptyBlock.
	p := DerivingParser empty.
	d := DelegateParser new parser: p.
	self assert: (e value: (ParserCompacter value: d) parser).! !

!DelegateParserTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:36'!
testDotLabelDescribesParser
	| p |
	p := self sampleParser.
	self assert: (p dotLabel includesSubString: 'Delegate').! !

!DelegateParserTest methodsFor: 'testing - generating' stamp: 'fbs 10/26/2012 19:01'!
testGenerateReturnsSubparsersParses
	| d p |
	p := $a asParser.
	d := DelegateParser new parser: p.
	self assert: #($a) equals: (d generate read: 1).
	self should: [d generate read: 2] raise: Incomplete.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 8/31/2012 12:12'!
testIsAsNullableAsSubparser
	| p |
	p := DelegateParser new parser: (EpsStar treeSet: {1}); yourself.
	self assert: p isNullable.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 9/11/2012 19:55'!
testIsDelegate
	self assert: self sampleParser isDelegate.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 10/24/2012 21:41'!
testIsNullable
	| p |
	p := DelegateParser new.
	p parser: $a asParser.
	self deny: (DerivingParser isNullableBlock value: p).! !

!DelegateParserTest methodsFor: 'testing - parsing' stamp: 'fbs 10/12/2012 18:51'!
testParseNullIsThatOfSubparser
	| p parseNull |
	parseNull := DerivingParser parseNullBlock.
	p := DelegateParser new
		parser: ((EpsStar treeSet: {1}) or: (EpsStar treeSet: {2}));
		yourself.
	self assert: #(1 2) asSet equals: (parseNull value: p).! !

!DelegateParserTest methodsFor: 'testing - accessing' stamp: 'fbs 8/31/2012 12:09'!
testParser
	| p t |
	t := $a asParser.
	p := DelegateParser new.
	p parser: t.
	self assert: p parser == t.! !

!DelegateParserTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:45'!
testPrintAsDigraphPrintsSubparsers
	| r digraph |
	r := $1 asParser.
	digraph := (DelegateParser new parser: r) printAsDigraph.
	self assert: (digraph includesSubString: r dotLabel) description: 'digraph doesn''t mention subparser'.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 9/12/2012 09:33'!
testResolvedDelegateIsResolved
	self assert: (DelegateParser new parser: $a asParser) isResolved.! !

!DelegateParserTest methodsFor: 'testing - evaluating' stamp: 'fbs 10/24/2012 21:38'!
testResolvedParserDoesntExecuteWhenNullableBlockWhenNotNullable
	| ranBlock |
	ranBlock := false.
	(DelegateParser new parser: $1 asParser)
		whenNullable: [:onlyParse | ranBlock := true ].
	self deny: ranBlock.! !

!DelegateParserTest methodsFor: 'testing - evaluating' stamp: 'fbs 10/24/2012 21:38'!
testResolvedParserExecutesWhenNullableBlockWhenNullable
	| ranBlock |
	ranBlock := false.
	(DelegateParser new parser: (EpsStar treeSet: {1}))
		whenNullable: [:onlyParse |
			ranBlock := true ].
	self assert: ranBlock.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 10/11/2012 09:32'!
testSubParserIsChildOfSelf
	| p |
	p := self sampleParser.
	self assert: (p parser isChildOf: p).! !

!DelegateParserTest methodsFor: 'testing - accessing' stamp: 'fbs 9/12/2012 09:27'!
testSubParsersOfUnresolvedDelegateIsEmpty
	| p |
	p := DelegateParser new.
	self assert:#() equals: p subParsers.! !

!DelegateParserTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:30'!
testSubParsersReturnsDelegatedParser
	| p |
	p := DelegateParser new.
	p parser: $a asParser.
	self assert: {p parser} equals: p subParsers.! !

!DelegateParserTest methodsFor: 'testing - testing' stamp: 'fbs 9/12/2012 09:32'!
testUnresolvedDelegateIsNotResolved
	self deny: DelegateParser new isResolved.! !

!DelegateParserTest methodsFor: 'testing - evaluating' stamp: 'fbs 10/24/2012 21:38'!
testUnresolvedParserNeverExecutesWhenNullableBlock
	| ranBlock |
	ranBlock := false.
	DelegateParser new whenNullable: [:onlyParse | ranBlock := true ].
	self deny: ranBlock.! !

!DerivingParserTestCase class methodsFor: 'as yet unclassified' stamp: 'fbs 8/30/2012 16:06'!
isAbstract
	^ self name = #DerivingParserTestCase! !

!DerivingParserTestCase methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:04'!
sampleParser
	self subclassResponsibility.! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 18:14'!
testAnythingOrEmptyIsItself
	| p |
	p := self sampleParser.
	self assert: (p or: DerivingParser empty) == p.! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 4/15/2013 22:08'!
testAnythingThenEmptyIsEmpty
	| p e empty cat |
	e := DerivingParser isEmptyBlock.
	empty := DerivingParser emptyExpected: 1 actual: 2.
	p := self sampleParser.
	cat := p then: empty.
	self assert: (e value: cat).
	self assert: 1 equals: cat expected.
	self assert: 2 equals: cat actual.! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 2/8/2013 08:42'!
testCompact
	| p |
	"Most parsers are already compact."
	p := self sampleParser.
	self assert: p equals: (ParserCompacter value: p).! !

!DerivingParserTestCase methodsFor: 'testing - deriving' stamp: 'fbs 4/13/2013 23:38'!
testDReturnsDerivative
	| d pn |
	d := DerivingParser deriverBlock.
	pn := DerivingParser parseNullBlock.
	self
		assert: (pn value: (d value: $1 value: self sampleParser))
		equals: (d value: $1 value: self sampleParser) parseNull.! !

!DerivingParserTestCase methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:35'!
testDotLabelDescribesParser
	| p |
	p := self sampleParser.
	self assert: (p dotLabel includesSubString: p className).! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 18:16'!
testEmptyOrAnythingIsItself
	| p |
	p := self sampleParser.
	self assert: (DerivingParser empty or: p) == p.! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 4/15/2013 22:08'!
testEmptyThenAnythingIsEmpty
	| e p |
	e := DerivingParser isEmptyBlock.
	p := self sampleParser.
	self assert: (e value: (DerivingParser empty then: p)).! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 8/30/2012 17:33'!
testForce
	| p |
	p := self sampleParser.
	self assert: p force == p.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:36'!
testIsCat
	self deny: self sampleParser isCat.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 21:14'!
testIsDelayed
	self deny: self sampleParser isDelayed.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 9/11/2012 19:55'!
testIsDelegate
	self deny: self sampleParser isDelegate.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 4/15/2013 21:14'!
testIsEmpty
	| e |
	e := DerivingParser isEmptyBlock.
	self deny: (e value: self sampleParser).! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 2/15/2013 09:30'!
testIsEmptyParser
	self deny: self sampleParser isEmptyParser.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:03'!
testIsEps
	self deny: self sampleParser isEps.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 9/4/2012 09:04'!
testIsEpsStar
	self deny: self sampleParser isEpsStar.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:29'!
testIsLazy
	self deny: self sampleParser isLazy.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:13'!
testIsLiteral
	self deny: self sampleParser isLiteral.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 10/24/2012 21:40'!
testIsNullable
	self deny: (DerivingParser isNullableBlock value: self sampleParser).! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:43'!
testIsRed
	self deny: self sampleParser isRed.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 9/13/2012 19:34'!
testIsStar
	self deny: self sampleParser isStar.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:48'!
testIsUnion
	self deny: self sampleParser isUnion.! !

!DerivingParserTestCase methodsFor: 'testing - testing' stamp: 'fbs 10/11/2012 09:30'!
testNeverIsChildOfSelf
	| p |
	p := self sampleParser.
	self deny: (p isChildOf: p).! !

!DerivingParserTestCase methodsFor: 'testing - parsing' stamp: 'fbs 4/17/2013 17:38'!
testOptionalMeansItAcceptsEmptyInput
	| p pn |
	pn := DerivingParser parseNullBlock.
	p := self sampleParser optional.
	self assert: ((pn value: p) add: #(); yourself) equals: (pn value: p).! !

DerivingParserTestCase subclass: #EmptyStringTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!EmptyStringTest methodsFor: 'fixtures' stamp: 'fbs 9/4/2012 09:01'!
sampleParser
	^ DerivingParser emptyString.! !

!EmptyStringTest methodsFor: 'testing - deriving' stamp: 'fbs 4/15/2013 22:08'!
testDerivativeIsEmpty
	| e |
	e := DerivingParser isEmptyBlock.
	self
		assert: (e value: (DerivingParser deriverBlock value: $a value: (DerivingParser emptyString)))
		description: 'd(_, eps*) == empty'! !

!EmptyStringTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:36'!
testDotLabelDescribesParser
	| p |
	p := self sampleParser.
	self assert: (p dotLabel includesSubString: 'eps').! !

!EmptyStringTest methodsFor: 'testing - generating' stamp: 'fbs 10/29/2012 12:03'!
testGenerateProducesOneEmptyString
	self assert: #(()) equals: (self sampleParser generate read: 1).
	self should: [self sampleParser generate read: 2] raise: Incomplete.! !

!EmptyStringTest methodsFor: 'testing - testing' stamp: 'fbs 9/4/2012 09:02'!
testIsEps
	self assert: self sampleParser isEps.! !

!EmptyStringTest methodsFor: 'testing - testing' stamp: 'fbs 10/24/2012 21:41'!
testIsNullable
	self assert: (DerivingParser isNullableBlock value: self sampleParser).! !

!EmptyStringTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:32'!
testSubParsersReturnsEmptySet
	self assert: {} equals: DerivingParser emptyString subParsers.! !

DerivingParserTestCase subclass: #EmptyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!EmptyTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:05'!
sampleParser
	^ DerivingParser empty.! !

!EmptyTest methodsFor: 'testing - parsing' stamp: 'fbs 2/18/2013 21:35'!
testAnythingThenEmptyIsEmpty
	| p e cat |
	e := DerivingParser emptyExpected: 1 actual: 2.
	p := self sampleParser.
	cat := p then: e.
	self assert: cat == p.! !

!EmptyTest methodsFor: 'testing - deriving' stamp: 'fbs 4/15/2013 22:09'!
testDerivativeIsItself
	| D e specialE |
	e := DerivingParser isEmptyBlock.
	D := DerivingParser deriverBlock.
	self assert: (e value: (D value: $a value: DerivingParser empty)).
	self assert: (D value: $a value: DerivingParser empty) == DerivingParser empty description: 'Exact empty not returned (unspecified empty)'.
	
	specialE := DerivingParser emptyExpected: 1 actual: 2.
	self assert: specialE = (D value: 3 value: specialE) description: 'Exact empty not returned (specified empty)'.! !

!EmptyTest methodsFor: 'testing - creating' stamp: 'fbs 2/14/2013 20:58'!
testEmptyMayHaveErrorMessage
	| e |
	e := Empty expected: 1 actual: 2.
	self assert: 'Expected a <1>, found a <2>' equals: e error.! !

!EmptyTest methodsFor: 'testing - generating' stamp: 'fbs 10/25/2012 18:43'!
testGenerateProducesNothing
	self should: [self sampleParser generate read: 1] raise: Incomplete.! !

!EmptyTest methodsFor: 'testing - testing' stamp: 'fbs 4/15/2013 21:17'!
testIsEmpty
	| e |
	e := DerivingParser isEmptyBlock.
	self assert: (e value: self sampleParser).! !

!EmptyTest methodsFor: 'testing - testing' stamp: 'fbs 2/15/2013 09:30'!
testIsEmptyParser
	self assert: self sampleParser isEmptyParser.! !

!EmptyTest methodsFor: 'testing - parsing' stamp: 'fbs 10/12/2012 18:52'!
testParseNullIsEmptySet
	| parseNull |
	parseNull := DerivingParser parseNullBlock.
	self assert: Set new equals: (parseNull value: DerivingParser empty).! !

!EmptyTest methodsFor: 'testing - parsing' stamp: 'fbs 2/15/2013 14:36'!
testReduceOfAnEmptyIsThatEmpty
	| e r |
	e := DerivingParser emptyExpected: 1 actual: 2.
	r := e reduce: #printString.
	self assert: e == r description: 'Red -> Empty => Empty'.! !

!EmptyTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:32'!
testSubParsersReturnsEmptySet
	self assert: {} equals: DerivingParser empty subParsers.! !

DerivingParserTestCase subclass: #EpsStarTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!EpsStarTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:23'!
sampleParser
	^ EpsStar treeSet: {$a}.! !

!EpsStarTest methodsFor: 'testing - deriving' stamp: 'fbs 4/15/2013 21:18'!
testDerivativeIsEmpty
	| e |
	e := DerivingParser isEmptyBlock.
	self
		assert: (e value: (DerivingParser deriverBlock value: $a value: (EpsStar treeSet: {})))
		description: 'd(_, eps*) == empty'! !

!EpsStarTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:37'!
testDotLabelDescribesParser
	| p |
	p := self sampleParser.
	self assert: (p dotLabel includesSubString: 'eps*').! !

!EpsStarTest methodsFor: 'testing - generating' stamp: 'fbs 10/25/2012 18:43'!
testGenerateProducesNothing
	self should: [self sampleParser generate read: 1] raise: Incomplete.! !

!EpsStarTest methodsFor: 'testing - testing' stamp: 'fbs 9/4/2012 09:04'!
testIsEpsStar
	self assert: self sampleParser isEpsStar.! !

!EpsStarTest methodsFor: 'testing - testing' stamp: 'fbs 10/24/2012 21:42'!
testIsNullable
	self assert: (DerivingParser isNullableBlock value: self sampleParser).! !

!EpsStarTest methodsFor: 'testing - parsing' stamp: 'fbs 10/12/2012 18:53'!
testParseNullIsTreeSet
	| parseNull |
	parseNull := DerivingParser parseNullBlock.
	self assert: {1. 2} asSet equals: (parseNull value: (EpsStar treeSet: {1. 2})).! !

!EpsStarTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:48'!
testPrintAsDigraphMentionsTreeSet
	| p digraph |
	p := EpsStar treeSet: {1}.
	digraph := p printAsDigraph.
	self assert: (digraph includesSubString: p treeSet printString).! !

!EpsStarTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:32'!
testSubParsersReturnsEmptySet
	self assert: {} equals: (EpsStar treeSet: {1}) subParsers.! !

DerivingParserTestCase subclass: #LiteralSetTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!LiteralSetTest methodsFor: 'fixtures' stamp: 'fbs 2/1/2013 08:53'!
sampleParser
	^ LiteralSet tokens: {$a. $b. $c}.! !

!LiteralSetTest methodsFor: 'testing - deriving' stamp: 'fbs 4/15/2013 21:18'!
testDerivativeOfLiteralWrtAnotherObjectIsEmpty
	| d e |
	e := DerivingParser isEmptyBlock.
	d := DerivingParser deriverBlock value: $d value: self sampleParser.
	self assert: (e value: d).
	self assert: 'Expected one of <$a, $b, $c>, found a <$d>' equals: d error.! !

!LiteralSetTest methodsFor: 'testing - deriving' stamp: 'fbs 2/1/2013 08:55'!
testDerivativeOfLiteralWrtSelfIsEps
	| d dLit litSet |
	d := DerivingParser deriverBlock.
	litSet := self sampleParser.
	self sampleParser tokens do: [:token |
		dLit := d value: token value: litSet.
		self assert: dLit isEpsStar.
		self assert: {token} asSet equals: dLit treeSet]! !

!LiteralSetTest methodsFor: 'testing - printing' stamp: 'fbs 2/18/2013 21:40'!
testDotLabelDescribesParser
	| p |
	p := self sampleParser.
	self assert: (p dotLabel includesSubString: 'LiteralSet').! !

!LiteralSetTest methodsFor: 'testing - generating' stamp: 'fbs 2/1/2013 17:28'!
testGenerateOnlyProducesAtMostNResults
	| p |
	p := self sampleParser.
	self should: [p generate read: (p tokens size + 1)] raise: Incomplete.! !

!LiteralSetTest methodsFor: 'testing - generating' stamp: 'fbs 2/1/2013 08:57'!
testGenerateProducesTheLiteral
	| p |
	p := self sampleParser.
	self assert: p tokens equals: (self sampleParser generate read: (p tokens size)).! !

!LiteralSetTest methodsFor: 'testing - testing' stamp: 'fbs 2/1/2013 08:53'!
testIsLiteral
	self assert: self sampleParser isLiteral.! !

!LiteralSetTest methodsFor: 'testing - parsing' stamp: 'fbs 2/1/2013 08:53'!
testParseNullIsEmptySet
	| parseNull |
	parseNull := DerivingParser parseNullBlock.
	self assert: Set new equals: (parseNull value: $a asParser).! !

!LiteralSetTest methodsFor: 'testing - printing' stamp: 'fbs 2/18/2013 21:41'!
testPrintAsDigraphMentionsTokens
	| p digraph |
	p := LiteralSet tokens: #(1 2).
	digraph := p printAsDigraph.
	self assert: (digraph includesSubString: p tokens printString).! !

!LiteralSetTest methodsFor: 'testing - accessing' stamp: 'fbs 2/1/2013 08:56'!
testSubParsersReturnsEmptySet
	self assert: {} equals: self sampleParser subParsers.! !

DerivingParserTestCase subclass: #LiteralTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!LiteralTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:06'!
sampleParser
	^ $a asParser.! !

!LiteralTest methodsFor: 'testing - deriving' stamp: 'fbs 4/15/2013 21:18'!
testDerivativeOfLiteralWrtAnotherObjectIsEmpty
	| d e |
	e := DerivingParser isEmptyBlock.
	d := DerivingParser deriverBlock value: $b value: $a asParser.
	self assert: (e value: d).
	self assert: 'Expected a <$a>, found a <$b>' equals: d error.! !

!LiteralTest methodsFor: 'testing - deriving' stamp: 'fbs 10/16/2012 16:59'!
testDerivativeOfLiteralWrtSelfIsEps
	| d |
	d := DerivingParser deriverBlock value: $a value: $a asParser.
	self assert: d isEpsStar.
	self assert: {$a} asSet equals: d treeSet.! !

!LiteralTest methodsFor: 'testing - generating' stamp: 'fbs 10/26/2012 18:54'!
testGenerateOnlyProducesOneResult
	self should: [self sampleParser generate read: 2] raise: Incomplete.! !

!LiteralTest methodsFor: 'testing - generating' stamp: 'fbs 10/26/2012 18:54'!
testGenerateProducesTheLiteral
	self assert: {self sampleParser token} equals: (self sampleParser generate read: 1).! !

!LiteralTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:13'!
testIsLiteral
	self assert: self sampleParser isLiteral.! !

!LiteralTest methodsFor: 'testing - parsing' stamp: 'fbs 10/12/2012 18:53'!
testParseNullIsEmptySet
	| parseNull |
	parseNull := DerivingParser parseNullBlock.
	self assert: Set new equals: (parseNull value: $a asParser).! !

!LiteralTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:32'!
testSubParsersReturnsEmptySet
	self assert: {} equals: $a asParser subParsers.! !

DerivingParserTestCase subclass: #RedTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!RedTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:44'!
sampleParser
	^ (EpsStar treeSet: {1. 2}) reduce: [:x | x + 1].! !

!RedTest methodsFor: 'testing - parsing' stamp: 'fbs 2/8/2013 08:46'!
testCompactOfNestedRedsComposesTheirReducers
	| c p1 p2 |
	p1 := 1 asParser reduce: [:i | i + 1].
	p2 := p1 reduce: [:j | j * 2].
	c := ParserCompacter value: p2.
	self assert: c isRed.
	self assert: c parser isLiteral.
	self assert: 4 equals: (c reducer value: 1).! !

!RedTest methodsFor: 'testing - parsing' stamp: 'fbs 2/8/2013 08:47'!
testCompactionIsRedOfCompactedSubparser
	| p c |
	p := DerivingParser deriverBlock value: $a value: ($a asParser star reduce: [:x | {$b}, x]).
	c := ParserCompacter value: p.
	self assert: c isRed.
	self assert: c parser == (ParserCompacter value: c parser).! !

!RedTest methodsFor: 'testing - deriving' stamp: 'fbs 10/16/2012 16:59'!
testDerivativeIsTheReductionOfTheSubParsersDerivative
	| d p sub parseNull |
	parseNull := DerivingParser parseNullBlock.
	sub := 1 asParser.
	p := sub reduce: [:x | x + 1].
	d := DerivingParser deriverBlock value: 1 value: p.
	self assert: d isRed.
	self assert: {2} asSet equals: (parseNull value: d) description: 'The subparser of the derivative is the derivative of the subparser'.
	self assert: d reducer == p reducer description: 'The reducer remains the same'.! !

!RedTest methodsFor: 'testing - generating' stamp: 'fbs 10/26/2012 18:59'!
testGenerateProducesReducedParsesOfTheSubparser
	| red |
	red := 1 asParser reduce: [:x | x + 1].
	self assert: #(2) equals: (red generate read: 1).
	self should: [red generate read: 2] raise: Incomplete.! !

!RedTest methodsFor: 'testing - testing' stamp: 'fbs 10/24/2012 21:42'!
testIsNullable
	"A reduction is as nullable as its subparser."
	| n |
	n := DerivingParser isNullableBlock.
	self assert: (n value: ((EpsStar treeSet: {1}) reduce: [:x | x + 1])).
	self deny: (DerivingParser isNullableBlock value: (DerivingParser empty reduce: [:x | x + 1])).! !

!RedTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:43'!
testIsRed
	self assert: self sampleParser isRed.! !

!RedTest methodsFor: 'testing - accessing' stamp: 'fbs 8/30/2012 22:11'!
testParser
	| p sub red |
	sub := $1 asParser.
	red := [:x | x + 1].
	p := sub reduce: red.
	self assert: sub equals: p parser.
	self assert: red equals: p reducer.! !

!RedTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:45'!
testPrintAsDigraphPrintsSubparsers
	| r digraph |
	r := $1 asParser.
	digraph := (r reduce: #yourself) printAsDigraph.
	self assert: (digraph includesSubString: r dotLabel) description: 'digraph doesn''t mention subparser'.! !

!RedTest methodsFor: 'testing - parsing' stamp: 'fbs 9/18/2012 09:38'!
testReduceOfaRedComposesReducers
	| r p |
	p := 1 asParser reduce: [:x | x + 1].
	r := p reduce: [:x | x * 2].
	self assert: r isRed.
	self assert: r parser isLiteral.
	self assert: 4 equals: (r reducer value: 1).! !

!RedTest methodsFor: 'testing - accessing' stamp: 'fbs 8/30/2012 22:11'!
testReducer
	| p sub red |
	sub := $1 asParser.
	red := [:x | x + 1].
	p := sub reduce: red.
	self assert: red equals: p reducer.! !

!RedTest methodsFor: 'testing - testing' stamp: 'fbs 10/11/2012 09:32'!
testSubParserIsChildOfSelf
	| p |
	p := self sampleParser.
	self assert: (p parser isChildOf: p).! !

!RedTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:33'!
testSubParsersReturnsSubParser
	| p |
	p := $a asParser reduce: #yourself.
	self assert: {p parser} equals: p subParsers.! !

DerivingParserTestCase subclass: #StarParserTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!StarParserTest methodsFor: 'fixtures' stamp: 'fbs 9/13/2012 19:34'!
sampleParser
	^ $a asParser star.! !

!StarParserTest methodsFor: 'testing - deriving' stamp: 'fbs 10/16/2012 16:58'!
testDerivativeOfStarParserPeelsOffOneCopyOfSubParser
	| p d |
	p := $a asParser star.
	d := DerivingParser deriverBlock value: $a value: p.
	self assert: d isCat.
	self assert: d first isEpsStar.
	self assert: p equals: d second.! !

!StarParserTest methodsFor: 'testing - generating' stamp: 'fbs 10/26/2012 09:28'!
testGenerateReturnsEverLongerParses
	self flag: #todo. "Really? A star parser is like empty | doitagain"
	self
		assert: #(() ($a) ($a $a) ($a $a $a) ($a $a $a $a))
		equals: ($a asParser star generate read: 5)! !

!StarParserTest methodsFor: 'testing - testing' stamp: 'fbs 9/13/2012 19:34'!
testIsStar
	self assert: self sampleParser isStar.! !

!StarParserTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:45'!
testPrintAsDigraphPrintsSubparsers
	| r digraph |
	r := $1 asParser.
	digraph := r star printAsDigraph.
	self assert: (digraph includesSubString: r dotLabel) description: 'digraph doesn''t mention subparser'.! !

!StarParserTest methodsFor: 'testing - testing' stamp: 'fbs 10/11/2012 09:32'!
testSubParserIsChildOfSelf
	| p |
	p := self sampleParser.
	self assert: (p parser isChildOf: p).! !

DerivingParserTestCase subclass: #UnionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!UnionTest methodsFor: 'fixtures' stamp: 'fbs 8/30/2012 16:52'!
sampleParser
	^ $a asParser or: $b asParser.! !

!UnionTest methodsFor: 'testing - deriving' stamp: 'fbs 4/15/2013 21:21'!
testCompactedEmptyUnionGivesDecentErrorMessage
	| c e u |
	"The result of having derived '1 asParser or: 2 asParser' with respect to 3."
	e := DerivingParser isEmptyBlock.
	u := Union
		left: (DerivingParser emptyExpected: 1actual: 3)
		right: (DerivingParser emptyExpected: 2 actual: 3).
	c := ParserCompacter value: u.
	self assert: (e value: c).
	self assert: #(1 2) equals: c expected.
	self assert: 3 equals: c actual.
	self assert: 'Expected one of <1, 2>, found a <3>' equals: c error.! !

!UnionTest methodsFor: 'testing - deriving' stamp: 'fbs 10/16/2012 17:05'!
testDerivativeIsUnionOfSubparserDerivatives
	| d |
	"This strange-looking grammar ensures that the derivative of both subparsers are not isEmpty."
	d := DerivingParser deriverBlock
		value: $a
		value: (($a asParser then: $c asParser) or: ($a asParser then: $b asParser)).
	self assert: d isUnion.
	"Left subparser is now 'we have read a $a and expect a $c next."
	self assert: d left isCat.
	self assert: d left first isEpsStar.
	self assert: d left second isLiteral.
	self assert: $c equals: d left second token.
	
	"Right subparser is now 'we have read a $a and expect a $b next."
	self assert: d right isCat.
	self assert: d right first isEpsStar.
	self assert: d right second isLiteral.
	self assert: $b equals: d right second token.! !

!UnionTest methodsFor: 'testing - generating' stamp: 'fbs 10/30/2012 17:30'!
testGenerateHandlesLeftSubparserWithInfiniteStream
	| u |
	"Left subparser an infinite number of parse trees; right can produce one."
	u := $a asParser star or: $c asParser.
	self assert: #(() $c ($a)) equals: (u generate read: 3).
	self shouldnt: [u generate read: 4] raise: Incomplete.! !

!UnionTest methodsFor: 'testing - generating' stamp: 'fbs 10/29/2012 20:15'!
testGenerateHandlesLeftSubparserWithShortStreams
	| u |
	"Left subparser can only produce one parse tree; right can produce two."
	u := $a asParser or: ($b asParser or: $c asParser).
	self assert: #($a $b $c) equals: (u generate read: 3).
	self should: [u generate read: 4] raise: Incomplete.! !

!UnionTest methodsFor: 'testing - generating' stamp: 'fbs 10/30/2012 17:31'!
testGenerateHandlesRightSubparserWithInfiniteStream
	| u |
	"Right subparser can an infinite number of parse trees; left can produce one."
	u := $a asParser or: $c asParser star.
	self assert: #($a () ($c)) equals: (u generate read: 3).
	self shouldnt: [u generate read: 4] raise: Incomplete.! !

!UnionTest methodsFor: 'testing - generating' stamp: 'fbs 10/29/2012 20:16'!
testGenerateHandlesRightSubparserWithShortStreams
	| u |
	"Right subparser can only produce one parse tree; left can produce two."
	u := ($a asParser or: $b asParser) or: $c asParser.
	self assert: #($a $c $b) equals: (u generate read: 3).
	self should: [u generate read: 4] raise: Incomplete.! !

!UnionTest methodsFor: 'testing - generating' stamp: 'fbs 10/26/2012 18:59'!
testGenerateProducesInterleavedParsesFromSubparsers
	| u |
	u := $a asParser or: $b asParser.
	self assert: #($a $b) equals: (u generate read: 2).
	self should: [u generate read: 3] raise: Incomplete.! !

!UnionTest methodsFor: 'testing - testing' stamp: 'fbs 8/30/2012 16:51'!
testIsUnion
	self assert: self sampleParser isUnion.! !

!UnionTest methodsFor: 'testing - accessing' stamp: 'fbs 8/31/2012 08:46'!
testLeft
	| p p1 p2 |
	p1 := $a asParser.
	p2 := $b asParser.
	p := p1 or: p2.
	self assert: p1 equals: p left.! !

!UnionTest methodsFor: 'testing - parsing' stamp: 'fbs 10/12/2012 18:53'!
testParseNullIsUnionOfSubParsersParseTrees
	| parseNull |
	parseNull := DerivingParser parseNullBlock.
	self assert: {1. 2} asSet equals: (parseNull value: ((EpsStar treeSet: {1}) or: (EpsStar treeSet: {2}))).! !

!UnionTest methodsFor: 'testing - printing' stamp: 'fbs 10/24/2012 09:43'!
testPrintAsDigraphPrintsSubparsers
	| l r digraph |
	l := $1 asParser.
	r := DerivingParser emptyString.
	digraph := (l or: r) printAsDigraph.
	self assert: (digraph includesSubString: l dotLabel) description: 'digraph doesn''t mention first subparser'.
	self assert: (digraph includesSubString: r dotLabel) description: 'digraph doesn''t mention second subparser'.! !

!UnionTest methodsFor: 'testing - accessing' stamp: 'fbs 8/31/2012 08:47'!
testRight
	| p p1 p2 |
	p1 := $a asParser.
	p2 := $b asParser.
	p := p1 or: p2.
	self assert: p2 equals: p right.! !

!UnionTest methodsFor: 'testing - testing' stamp: 'fbs 10/11/2012 09:31'!
testSubParserIsChildOfUnion
	| p |
	p := self sampleParser.
	self assert: (p left isChildOf: p).
	self assert: (p right isChildOf: p).! !

!UnionTest methodsFor: 'testing - accessing' stamp: 'fbs 9/5/2012 19:27'!
testSubParsersReturnsLeftAndRight
	| p |
	p := $a asParser or: $b asParser.
	self assert: {p left. p right} equals: p subParsers.! !

!UnionTest methodsFor: 'testing - parsing' stamp: 'fbs 2/8/2013 08:47'!
testUnionOfCompactedParsersCompactsToItself
	| u |
	u := $a asParser or: $b asParser.
	self assert: u == (ParserCompacter value: u).! !

TestCase subclass: #GeneratingTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!GeneratingTest methodsFor: 'testing - generating' stamp: 'fbs 10/31/2012 14:59'!
testRightRecursiveGrammar
	| ones d g |
	d := DelegateParser new.
	ones := DerivingParser emptyString or: (1 asParser then: d).
	d parser: ones.
	g := ones generate.
	self assert: #(()) equals: (g read: 1).
	self assert: #((1 ())) equals: (g read: 1).
	self assert: #((1 (1 ()))) equals: (g read: 1).! !

TestCase subclass: #LeastFixedPointTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!LeastFixedPointTest methodsFor: 'testing - lfp' stamp: 'fbs 8/9/2012 17:29'!
testLfpMemoises
	| f run |
	run := false.
	f := [:x | run := true. x*x - 2] lfpWithBottom: 0.
	f value: 0.
	run := false. "Reset the flag, because the block _will_ have been run in the previous statements."
	f value: 0.
	"The block should not be executed again."
	self deny: run.! !

!LeastFixedPointTest methodsFor: 'testing - lfp' stamp: 'fbs 7/13/2012 18:57'!
testLfpOfIdentityIsOwnThing
	| f |
	f := LeastFixedPoint run: #yourself bottom: 1.
	self assert: 1 equals: (f value: 1).
	self assert: 2 equals: (f value: 2).! !

!LeastFixedPointTest methodsFor: 'testing - lfp' stamp: 'fbs 8/9/2012 17:31'!
testLfpReturnsLeastFixedPoint
	| f |
	"This example doesn't work because lfp really does this: 'run the block until its value stops changing'. Given the simple and pure function below, that's immediately true, and the 'its value stops changing' value is just the first result: f(0) = -2."
	f := [:x | x*x - 2] lfpWithBottom: 0.
	"f(0) = -2. f(-2) = 2. f(2) = 2. So f(f(f(x))) = x when x = 2."
	f value: 0.
	"self assert: 2 equals: fixp."! !

!LeastFixedPointTest methodsFor: 'testing - lfp' stamp: 'fbs 8/30/2012 21:07'!
testLfpWithBottomExplainsCorrectArgCount
	[#+ lfpWithBottom: 0]
		on: ArityMismatch
		do: [:am |
			self assert: 2 equals: am actualArgCount.
			self assert: 1 equals: am expectedArgCount].! !

TestCase subclass: #ParseNullTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParseNullTest methodsFor: 'as yet unclassified' stamp: 'fbs 10/23/2012 09:42'!
testEmptyStringIsEmptyString
	| parseNull |
	parseNull := DerivingParser parseNullBlock.
	self assert: #(()) asSet equals: (parseNull value: DerivingParser emptyString).! !

!ParseNullTest methodsFor: 'as yet unclassified' stamp: 'fbs 10/19/2012 20:57'!
testParseNullOfEpsStarIsItsTreeSet
	| p |
	p := EpsStar treeSet: {1. 2}.
	self assert: p treeSet equals: (DerivingParser parseNullBlock value: p).! !

TestCase subclass: #ParseTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!ParseTests methodsFor: 'testing - parsers' stamp: 'fbs 8/30/2012 16:49'!
testOrCreatesAUnionParser
	self assert: ($a asParser or: $b asParser) isUnion.! !

!ParseTests methodsFor: 'testing - parsers' stamp: 'fbs 10/19/2012 21:16'!
testRightRecursiveGrammar
	| ones d D Dones DDones parseNull |
	D := DerivingParser deriverBlock.
	parseNull := DerivingParser parseNullBlock.
	d := DelegateParser new.
	ones := DerivingParser emptyString or: (1 asParser then: d).
	d parser: ones.
	Dones := (D value: 1 value: ones).
	DDones := (D value: 1 value: Dones).
	self assert: #((1)) asSet equals: (parseNull value: Dones).
	self assert: #((1 1)) asSet equals: (parseNull value: DDones).! !

!ParseTests methodsFor: 'testing - parsers' stamp: 'fbs 8/30/2012 16:32'!
testThenCreatesACatParser
	self assert: ($a asParser then: $b asParser) isCat.! !

TestCase subclass: #ParserCompacterTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:44'!
testCanCompactCatWithCyclicSubparser
"	| c d p sub |
	d := DelegateParser new.
	sub := DerivingParser emptyString or: (1 asParser then: d).
	d parser: sub.
	p := Cat
			with: DerivingParser empty delay
			with: d.
	c := ParserCompacter value: p.
	
	self assert: c isDelegate."! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:43'!
testCanCompactDelegateContainingCyclicParsers
	| ones c d D Dones DDones parseNull |
	D := DerivingParser deriverBlock.
	parseNull := DerivingParser parseNullBlock.
	d := DelegateParser new.
	ones := DerivingParser emptyString or: (1 asParser then: d).
	d parser: ones.
	Dones := (D value: 1 value: ones).
	DDones := (D value: 1 value: Dones).
	c := ParserCompacter value: DDones.
	self assert: c isRed.
	self assert: #((1 1)) asSet equals: c parseNull.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:43'!
testCanCompactNestedReds
	| c p |
	p := ((EpsStar treeSet: {2}) reduce: [:x | x + 1]) reduce: [:y | y * 2].
	c := ParserCompacter value: p.
	self assert: c isRed.
	self assert: c parser isEpsStar.
	self assert: #(6) asSet equals: c parseNull.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:43'!
testCanCompactRedDelegateRed
	| c p |
	p := ((DelegateParser new parser: ((EpsStar treeSet: {1}) reduce: [:x | x + 2])) reduce: [:x | x + 1]).
	c := ParserCompacter value: p.
	self assert: c isRed.
	self assert: c parser isEpsStar.
	self assert: #(4) asSet equals: c parseNull.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 4/15/2013 21:20'!
testCanCompactRedEmpty
	| c e p |
	"Red -> Empty => Empty"
	e := DerivingParser isEmptyBlock.
	p := (DerivingParser emptyExpected: 1 actual: 2) reduce: #yourself.
	c := ParserCompacter value: p.
	self assert: (e value: c).
	self assert: 1 equals: c expected.
	self assert: 2 equals: c actual.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 4/15/2013 21:20'!
testCanCompactStarEmpty
	| c e p |
	"Empty star -> Empty"
	e := DerivingParser isEmptyBlock.
	p := (DerivingParser emptyExpected: 1 actual: 2) star.
	c := ParserCompacter value: p.
	self assert: (e value: c).
	self assert: 1 equals: c expected.
	self assert: 2 equals: c actual.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:43'!
testCanCompactUnionDeeply
	| c p |
	"#then: and #or: are smart about dropping empty parsers. Since these may occur during derivation, construct the Cats and Unions by hand."
	p := Union left: DerivingParser empty delay right: (Union left: $a asParser delay right: (Cat with: DerivingParser empty delay with: $b asParser delay) delay) delay.
	c := ParserCompacter value: p.
	self assert: c isLiteral.
	self assert: $a equals: c token.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:44'!
testCanCompactUnionWithCyclicSubparser
"	| c d p sub |
	d := DelegateParser new.
	sub := DerivingParser emptyString or: (1 asParser then: d).
	d parser: sub.
	p := 2 asParser
		or: (Cat
			with: DerivingParser empty delay
			with: d).
	c := ParserCompacter value: p.
	
	self assert: c isLiteral.
	self assert: 2 equals: c token."! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 4/15/2013 21:20'!
testCompactedCatParsersPreserveErrorInformation
	| c e p |
	e := DerivingParser isEmptyBlock.
	p := Cat with: (DerivingParser emptyExpected: 'expected' actual: 'actual') with: ($2 asParser).
	c := ParserCompacter value: p.
	self assert: (e value: c) description: 'Compacted parser not empty'.
	self assert: 'expected' equals: c expected.
	self assert: 'actual' equals: c actual.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 4/15/2013 21:20'!
testCompactedCatParsersPreserveErrorInformationWithTrailingEmptyParser
	| c e p |
	e := DerivingParser isEmptyBlock.
	p := Cat with: ($2 asParser) with: (DerivingParser emptyExpected: 'expected' actual: 'actual').
	c := ParserCompacter value: p.
	self assert: (e value: c) description: 'Compacted parser not empty'.
	self assert: 'expected' equals: c expected.
	self assert: 'actual' equals: c actual.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:44'!
testCompactionDiscardsEmptyUnionSubparsers
	| c p |
	p := Union
		left: (Cat
			with: (EpsStar treeSet: #(1)) delay
			with: (Cat
				with: (EpsStar treeSet: #(2)) delay
				with: DerivingParser emptyString delay) delay)
		right: (Cat "This subparse is discarded."
			with: DerivingParser empty delay
			with: DerivingParser emptyString delay) delay.
	c := ParserCompacter value: p.
	"Because of the EpsStar, the Cat is reduced to a Red nesting the Cat's right subparser."
	self assert: c isRed.
	self assert: c parser isEps.
	self assert: #((1 2)) asSet equals: (DerivingParser parseNullBlock value: c).! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:44'!
testCompactionOfAnythingOrEmptyIsAnything
	self assert: (ParserCompacter value: ($a asParser or: DerivingParser empty)) isLiteral.
	self assert: (ParserCompacter value: ((EpsStar treeSet: {1}) or: DerivingParser empty)) isEpsStar.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 4/15/2013 21:20'!
testCompactionOfAnythingThenEmptyIsEmpty
	| e |
	e := DerivingParser isEmptyBlock.
	self assert: (e value: (ParserCompacter value: ($a asParser then: DerivingParser empty))).
	self assert: (e value: (ParserCompacter value: ((EpsStar treeSet: {1}) then: DerivingParser empty))).! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:44'!
testCompactionOfEmptyOrAnythingIsAnything
	self assert: (ParserCompacter value: (DerivingParser empty or: $a asParser)) isLiteral.
	self assert: (ParserCompacter value: (DerivingParser empty or: (EpsStar treeSet: {1}))) isEpsStar.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 4/15/2013 21:21'!
testCompactionOfEmptyThenAnythingIsEmpty
	| e |
	e := DerivingParser isEmptyBlock.
	self assert: (e value: (ParserCompacter value: (DerivingParser empty then: $a asParser))).
	self assert: (e value: (ParserCompacter value: (DerivingParser empty then: (EpsStar treeSet: {1})))).! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:44'!
testCompactionRemovesUnnecessaryDelegates
	"If a delegate doesn't form part of a cycle, it does no harm to point to the delegate's reference instead of the delegate. This also makes the parser one node smaller, potentially."
	| c p |
	p := $a asParser then: (DelegateParser new parser: $b asParser).
	c := ParserCompacter value: p.
	self assert: c isCat.
	self assert: c second isLiteral.! !

!ParserCompacterTest methodsFor: 'testing' stamp: 'fbs 2/15/2013 14:24'!
testOnes
	| c d ones DDones D |
	D := DerivingParser deriverBlock.
	d := DelegateParser new.
	"This parser differs from testCanCompactDelegateContainingCyclicParsers' in that it's 'middle recursive'."
	ones := DerivingParser emptyString or: (d then: 1 asParser).
	d parser: ones.
	DDones := D value: 1 value: (D value: 1 value: ones).
	c := ParserCompacter value: DDones.
"	self assert: c isRed.
	self assert: c parser isDelegate."
	self assert: #((1 1)) asSet equals: c parseNull.! !

TestCase subclass: #ParserInspectorTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Test'!

!ParserInspectorTest methodsFor: 'testing - accessing' stamp: 'fbs 10/24/2012 21:45'!
testFieldListIncludesGraphItem
	| i |
	i := ParserInspector inspect: DerivingParser empty.
	self assert: (i fieldList includes: '<graph>').! !

!ParserInspectorTest methodsFor: 'testing - accessing' stamp: 'fbs 10/24/2012 21:50'!
testHandlesDisplayingBrokenGraph
	| i |
	i := ParserInspector inspect: (DerivingParser empty then: nil).
	self shouldnt: [i toggleIndex: (i fieldList size).] raise: Error.! !

!ParserInspectorTest methodsFor: 'testing - accessing' stamp: 'fbs 10/24/2012 21:49'!
testSelectingGraphPrintsGraph
	| i |
	i := ParserInspector inspect: DerivingParser empty.
	i toggleIndex: (i fieldList size).
	self assert: i object printAsDigraph printString equals: i contents asString.! !

TestCase subclass: #ParserWalkerTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParserWalkerTestCase class methodsFor: 'testing' stamp: 'fbs 9/8/2012 23:21'!
isAbstract
	^ self == ParserWalkerTestCase.! !

!ParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/8/2012 23:19'!
testParserWalkerVisitsAllSubparsers
	| p |
	p := $a asParser then: ($b asParser or: $c asParser).
	self assert: 5 equals: (self walker walk: p collecting: #yourself) size.! !

!ParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/8/2012 23:19'!
testWalkCollectingReturnsMappedValues
	| map p |
	p := $a asParser then: ($b asParser or: $c asParser).
	map := self walker walk: p collecting: #className.
	self assert: #('Union' 'Cat' 'Literal' 'Literal' 'Literal') sorted equals: map sorted.! !

!ParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/8/2012 23:19'!
testWalkIsCycleSafe
	| p d |
	d := DelegateParser new.
	p := ($a asParser then: d) or: DerivingParser emptyString.
	d parser: p.
	self assert: 5 equals: (self walker walk: p collecting: #yourself) size.! !

!ParserWalkerTestCase methodsFor: 'fixtures' stamp: 'fbs 9/8/2012 23:19'!
walker
	self subclassResponsibility.! !

ParserWalkerTestCase subclass: #PostOrderParserWalkerTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!PostOrderParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/7/2012 17:39'!
testParserWalkerUsesPostorderTraversal
	| p |
	p := $a asParser then: ($b asParser or: $c asParser).
	self assert: #('Literal' 'Literal' 'Literal' 'Union' 'Cat') equals: (PostOrderParserWalker walk: p collecting: #className).! !

!PostOrderParserWalkerTestCase methodsFor: 'fixtures' stamp: 'fbs 9/8/2012 23:19'!
walker
	^ PostOrderParserWalker.! !

ParserWalkerTestCase subclass: #PreOrderParserWalkerTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!PreOrderParserWalkerTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 9/7/2012 17:36'!
testParserWalkerUsesPreorderTraversal
	| p |
	p := $a asParser then: ($b asParser or: $c asParser).
	self assert: #('Cat' 'Literal' 'Union' 'Literal' 'Literal') equals: (ParserWalker walk: p collecting: #className).! !

!PreOrderParserWalkerTestCase methodsFor: 'fixtures' stamp: 'fbs 9/8/2012 23:20'!
walker
	^ PreOrderParserWalker.! !

TestCase subclass: #ParsingBlockClosureExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingBlockClosureExtensionTest methodsFor: 'helpers' stamp: 'fbs 7/17/2012 20:23'!
assert: aBlock caches: aSelector withArguments: someObjects
	| b run |
	run := false.
	b := [:x | run := true. aBlock value: x] memoized.
	b perform: aSelector withArguments: someObjects.
	run := false.
	b perform: aSelector withArguments: someObjects.
	self deny: run.
	b perform: aSelector withArguments: (someObjects collect: [:x | x + 1]).
	self assert: run.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing' stamp: 'fbs 8/30/2012 21:08'!
testArityMatchesNumArgs
	self assert: 0 equals: [] arity.
	self assert: 1 equals: [:x | x] arity.
	self assert: 2 equals: [:x :y | x + y] arity.
	self assert: 4 equals: [:x :y :z :w | x + y + z + w] arity.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing' stamp: 'fbs 8/30/2012 21:13'!
testAsParserReturnsDelayedParser
	self assert: [1] asParser isDelayed.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing' stamp: 'fbs 7/20/2012 22:05'!
testLfpRequiresUnaryBlock
	self should: [[1] lfpWithBottom: 0] raise: ArityMismatch.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing' stamp: 'fbs 7/13/2012 18:50'!
testLfpReturnsALeastFixedPoint
	self assert: LeastFixedPoint equals: ([:x | x ] lfpWithBottom: 0) class.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 20:20'!
testMemoizedBlockCachesResults
	self assert: [:x | x ] caches: #value: withArguments: #(1).! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 20:22'!
testMemoizedBlockCachesResultsFromValueValue
	| b run |
	b := [:x :y | run := true. x + y] memoized.
	b value: 1 value: 2.
	run := false.
	b value: 1 value: 2.
	self deny: run.
	b value: 1 value: 1.
	self assert: run.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 20:22'!
testMemoizedBlockCachesResultsFromValueValueValue
	| b run |
	b := [:x :y :z | run := true. x + y + z] memoized.
	b value: 1 value: 2 value: 3.
	run := false.
	b value: 1 value: 2 value: 3.
	self deny: run.
	b value: 1 value: 2 value: 1.
	self assert: run.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 20:22'!
testMemoizedBlockCachesResultsFromValueValueValueValue
	| b run |
	b := [:x :y :z :w | run := true. x + y + z + w] memoized.
	b value: 1 value: 2 value: 3 value: 4.
	run := false.
	b value: 1 value: 2 value: 3 value: 4.
	self deny: run.
	b value: 1 value: 2 value: 3 value: 1.
	self assert: run.! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 18:46'!
testMemoizedBlockReturnsExpectedValue
	| b |
	b := [:x | x] memoized.

	self assert: 1 equals: (b value: 1).
	self assert: 2 equals: (b value: 2)! !

!ParsingBlockClosureExtensionTest methodsFor: 'testing - memoizing' stamp: 'fbs 7/17/2012 18:44'!
testMemoizedReturnsMemoizingBlock
	self assert: ([:x | x] memoized isKindOf: MemoizingBlock).! !

TestCase subclass: #ParsingCollectionExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingCollectionExtensionTest methodsFor: 'testing' stamp: 'fbs 10/19/2012 09:49'!
testCartesianProductWithEmptyLeftReturnsEmpty
	self assert: Set new equals: (Set new cartesianProduct: #+ with: #(1) asSet).
	self assert: #() equals: (#() cartesianProduct: #+ with: #(1)).
	self assert: OrderedCollection new equals: (OrderedCollection new cartesianProduct: #+ with: (OrderedCollection with: 1)).! !

!ParsingCollectionExtensionTest methodsFor: 'testing' stamp: 'fbs 10/19/2012 09:49'!
testCartesianProductWithEmptyRightReturnsEmpty
	self assert: Set new equals: (#(1) asSet cartesianProduct: #+ with: Set new).
	self assert: #() equals: (#(1) cartesianProduct: #+ with: #()).
	self assert: OrderedCollection new equals: ((OrderedCollection with: 1) cartesianProduct: #+ with: OrderedCollection new).! !

!ParsingCollectionExtensionTest methodsFor: 'testing' stamp: 'fbs 9/5/2012 09:16'!
testCartesianProductWithNonEmptyCollections
	"We show the duplicate 5 to explain the workings of the product. As a Set, the duplicate will be removed."
	self assert: #(4 5 5 6) asSet equals: (#(1 2) asSet cartesianProduct: #+ with: #(3 4) asSet).
	self assert: #(4 5 5 6) equals: (#(1 2) cartesianProduct: #+ with: #(3 4)).
	self assert: #(4 5 5 6) asOrderedCollection equals: (#(1 2) asOrderedCollection cartesianProduct: #+ with: #(3 4) asOrderedCollection).! !

TestCase subclass: #ParsingObjectExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingObjectExtensionTest methodsFor: 'testing - instance creation' stamp: 'fbs 10/24/2012 09:33'!
testAsArray
	self assert: #(1) equals: 1 asArray.
	self assert: #(1) equals: #(1) asArray.! !

!ParsingObjectExtensionTest methodsFor: 'testing - instance creation' stamp: 'fbs 8/30/2012 16:07'!
testAsParserReturnsLiteral
	self assert: $a asParser isLiteral.
	self assert: 1 asParser isLiteral.
	self assert: Object new asParser isLiteral.! !

TestCase subclass: #ParsingSequenceableCollectionExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingSequenceableCollectionExtensionTest methodsFor: 'as yet unclassified' stamp: 'fbs 4/15/2013 21:21'!
testAsSequenceParserOfEmptyCollectionReturnsEmpty
	| e |
	e := DerivingParser isEmptyBlock.
	self assert: (e value: #() asSequenceParser).! !

!ParsingSequenceableCollectionExtensionTest methodsFor: 'as yet unclassified' stamp: 'fbs 4/15/2013 21:02'!
testAsSequenceParserReturnsaCatParserOnCollectionElements
	| pn p |
	pn := DerivingParser parseNullBlock.
	p := ('nil' reading parsing: ('nil' asSequenceParser)) rest last.
	self assert: #($n $i $l) equals: (pn value: p) anyOne.! !

TestCase subclass: #ParsingSymbolExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingSymbolExtensionTest methodsFor: 'testing' stamp: 'fbs 8/30/2012 21:06'!
testLfpRequiresUnarySymbol
	self should: [#+ lfpWithBottom: 0] raise: ArityMismatch.! !

TestCase subclass: #ParsingXTReadStreamExtensionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!ParsingXTReadStreamExtensionTest methodsFor: 'testing' stamp: 'fbs 10/31/2012 13:26'!
testInterleavingEmptyStreamsRaisesIncomplete
	self should: [(#() reading interleave: #() reading) read: 1] raise: Incomplete.! !

!ParsingXTReadStreamExtensionTest methodsFor: 'testing' stamp: 'fbs 10/31/2012 13:09'!
testInterleavingInfiniteStreamsIsFair
	self assert: #(1 2 1 2 1 2 1) equals: (([1] reading interleave: [2] reading) read: 7).! !

!ParsingXTReadStreamExtensionTest methodsFor: 'testing' stamp: 'fbs 10/31/2012 13:10'!
testInterleavingPermitsLeftStreamToStarve
	self assert: #(1 2 2) equals: ((#(1) reading interleave: [2] reading) read: 3).! !

!ParsingXTReadStreamExtensionTest methodsFor: 'testing' stamp: 'fbs 10/31/2012 13:10'!
testInterleavingPermitsRightStreamToStarve
	self assert: #(1 2 1) equals: (([1] reading interleave: #(2) reading) read: 3).! !

!ParsingXTReadStreamExtensionTest methodsFor: 'testing' stamp: 'fbs 4/15/2013 20:59'!
testParsingAutomaticallyTracksPosition
	| pn |
	pn := DerivingParser parseNullBlock.
	self
		assert: #(#(#($a)) #(#($a $a)) #(#($a $a $a))) "Each element is a parse _forest_."
		equals: (('aaa' reading parsing: $a asParser star) rest collect: [:parser | (pn value: parser) asArray]).! !

TestCase subclass: #UnresolvedDelegateTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!UnresolvedDelegateTest methodsFor: 'testing' stamp: 'fbs 10/18/2012 09:23'!
testCanRaiseThrougConvenienceMethod
	self
		should: [UnresolvedDelegate accessing: DerivingParser empty]
		raise: UnresolvedDelegate.! !

!UnresolvedDelegateTest methodsFor: 'testing' stamp: 'fbs 10/18/2012 09:22'!
testCanStoreParserReference
	| p e |
	p := DerivingParser empty.
	e := UnresolvedDelegate new parser: p.
	self assert: p equals: e parser.! !

!UnresolvedDelegateTest methodsFor: 'testing' stamp: 'fbs 10/18/2012 09:25'!
testExposesUnresolvedDelegate
	[UnresolvedDelegate accessing: DerivingParser empty]
		on: UnresolvedDelegate
		do: [:e | self assert: DerivingParser empty equals: e parser].! !

TestCase subclass: #XTPositionTrackingStreamTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support-Tests'!

!XTPositionTrackingStreamTest methodsFor: 'testing - column numbers' stamp: 'fbs 2/11/2013 21:12'!
testColumnNumberIncrementsAfterReading
	| stream |
	stream := XTPositionTrackingStream on: '123' reading.
	stream get.
	self assert: 1 equals: stream columnNumber description: 'After 1 get'.
	stream get.
	self assert: 2 equals: stream columnNumber description: 'After 2 gets'.
	stream get.
	self assert: 3 equals: stream columnNumber description: 'After 3 gets'.! !

!XTPositionTrackingStreamTest methodsFor: 'testing - column numbers' stamp: 'fbs 2/11/2013 21:06'!
testColumnNumberStartsAtZero
	self assert: 0 equals: (XTPositionTrackingStream on: '' reading) columnNumber.! !

!XTPositionTrackingStreamTest methodsFor: 'testing - column numbers' stamp: 'fbs 2/11/2013 22:20'!
testColumnNumbersResetAfterLineFeed
	| stream |
	stream := XTPositionTrackingStream on: ('1\2\3' replaceAll: $\ with: Character lf) reading.
	stream get.
	self assert: 1 equals: stream columnNumber description: 'After 1 get'.
	stream get.
	self assert: 0 equals: stream columnNumber description: 'After lf get'.
	stream get.
	self assert: 1 equals: stream columnNumber description: 'Line 2, after 1 get'.! !

!XTPositionTrackingStreamTest methodsFor: 'testing - line numbers' stamp: 'fbs 2/11/2013 21:18'!
testLineNumberDoesntIncrementOnOrdinaryCharacters
	| stream |
	stream := XTPositionTrackingStream on: '123' reading.
	stream get.
	self assert: 1 equals: stream lineNumber description: 'After 1 get'.
	stream get.
	self assert: 1 equals: stream lineNumber description: 'After 2 gets'.
	stream get.
	self assert: 1 equals: stream lineNumber description: 'After 3 gets'.! !

!XTPositionTrackingStreamTest methodsFor: 'testing - line numbers' stamp: 'fbs 2/11/2013 21:22'!
testLineNumberIncrementsAfterReadingaNewline
	| stream |
	stream := XTPositionTrackingStream on: ('1\2\3' replaceAll: $\ with: Character lf) reading.
	stream get.
	self assert: 1 equals: stream lineNumber description: 'After 1 get'.
	stream get.
	self assert: 2 equals: stream lineNumber description: 'After 1 lf get'.
	stream get.
	self assert: 2 equals: stream lineNumber description: 'Line 2, 1 get'.
	stream get.
	self assert: 3 equals: stream lineNumber description: 'After 2 lf gets'.
	stream get.
	self assert: 3 equals: stream lineNumber description: 'Line 3, 1 get'.! !

!XTPositionTrackingStreamTest methodsFor: 'testing - line numbers' stamp: 'fbs 2/11/2013 21:06'!
testLineNumberStartsAtZero
	self assert: 0 equals: (XTPositionTrackingStream on: '' reading) lineNumber.! !

!XTPositionTrackingStreamTest methodsFor: 'testing - parsing' stamp: 'fbs 4/11/2013 22:20'!
testParsingParsesWithCatParsers
	| d pn digits |
	d := DerivingParser deriverBlock.
	pn := DerivingParser parseNullBlock.
	digits := 1 asParser then: 2 asParser.
	self assert: Set new equals: (pn value: (d value: $n value: digits)).
	self assert: #(1 2) equals: (pn value: (d value: 2 value: (d value: 1 value: digits))) anyOne.! !

!XTPositionTrackingStreamTest methodsFor: 'testing - parsing' stamp: 'fbs 4/11/2013 22:06'!
testParsingParsesWithSimpleParsers
	| d pn |
	d := DerivingParser deriverBlock.
	pn := DerivingParser parseNullBlock.
	self assert: $n equals: (pn value: (d value: $n value: $n asParser)) anyOne.
	self assert: 1 equals: (pn value: (d value: 1 value: 1 asParser)) anyOne.
	self assert: false equals: (pn value: (d value: false value: false asParser)) anyOne.! !

!XTPositionTrackingStreamTest methodsFor: 'testing - instance creation' stamp: 'fbs 2/18/2013 21:48'!
testPositionTrackingAddsTrackingToPlainStreams
	| s |
	s := '123' reading positionTracking.
	s get.
	self assert: 1 equals: s columnNumber.
	self assert: 1 equals: s lineNumber.! !

WeakValueDictionary subclass: #WeakDictionary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!WeakDictionary commentStamp: 'fbs 7/20/2012 22:17' prior: 0!
I am like a WeakValueDictionary, but I regard nil values as meaning "I don't have this thing".

That means that if a parser p is garbage collected, myself includesKey: p will return false. Thus, I look much like a normal Dictionary to my users.!

!WeakDictionary methodsFor: 'accessing' stamp: 'fbs 7/20/2012 19:00'!
at: key
	^ (super at: key)
		ifNil: [self errorKeyNotFound: key]! !

!WeakDictionary methodsFor: 'testing' stamp: 'fbs 7/24/2012 11:54'!
includesKey: anObject
	^ [(super includesKey: anObject) and: [(self at: anObject) notNil]]
		on: KeyNotFound do: [false]! !

XTReadStream subclass: #DelayedStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

DelayedStream class
	instanceVariableNames: 'source'!

!DelayedStream commentStamp: 'fbs 10/30/2012 11:33' prior: 0!
A DelayedStream is an otherwise normal XTReadStream except that it reads off a source that has not yet been calculated:

	hasRun := false.
	delay := DelayedStream on: [hasRun := true. #(1)].
	self deny: hasRun.
	self assert: 1 equals: (delay read: 1).
	self assert: hasRun.!

DelayedStream class
	instanceVariableNames: 'source'!

!DelayedStream methodsFor: 'initialize-release' stamp: 'fbs 10/30/2012 13:37'!
contentsSpecies
	^ self force contentsSpecies.! !

!DelayedStream methodsFor: 'private' stamp: 'fbs 10/31/2012 08:49'!
force
	source isBlock ifTrue: [source := source value reading].
	^ source.! !

!DelayedStream methodsFor: 'accessing' stamp: 'fbs 10/30/2012 13:37'!
read: anInteger into: aSequenceableCollection at: startIndex
	^ self force read: anInteger into: aSequenceableCollection at: startIndex.! !

XTReadStream subclass: #XTPositionTrackingStream
	instanceVariableNames: 'columnNumber lineNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!XTPositionTrackingStream commentStamp: 'fbs 2/11/2013 21:24' prior: 0!
I keep track of column/line numbers. Initially these are set to 0. After reading the first character I am at position 1/1 - the first character of the first line. Every time I read a linefeed character my line number increments and character number resets to zero.!

!XTPositionTrackingStream methodsFor: 'accessing' stamp: 'fbs 2/11/2013 21:13'!
columnNumber
	^ columnNumber.! !

!XTPositionTrackingStream methodsFor: 'initialize-release' stamp: 'fbs 2/11/2013 21:11'!
contentsSpecies
	^ source contentsSpecies.! !

!XTPositionTrackingStream methodsFor: 'initialize-release' stamp: 'fbs 2/11/2013 21:19'!
initialize
	super initialize.
	columnNumber := 0.
	lineNumber := 0.! !

!XTPositionTrackingStream methodsFor: 'accessing' stamp: 'fbs 2/11/2013 21:19'!
lineNumber
	^ lineNumber.! !

!XTPositionTrackingStream methodsFor: 'parsing' stamp: 'fbs 4/12/2013 22:12'!
parsing: aParser
	"Return an XTReadStream that takes a parser aParser, consumes a character C, calculates the derivative of aParser with respect to C, and produces a new parser. Throws a ParseError if it unexpectedly derives Empty."
	| d derivative parse |
	derivative := aParser.
	d := DerivingParser deriverBlock.
	^ (self transforming: [:input :output |
		parse := [:next :p | | drv |
			drv := ParserCompacter value: (d value: next value: p).
			drv isEmptyParser
				ifTrue: [ParseError signalLine: input lineNumber column: input columnNumber error: drv error]
				ifFalse: [drv]].
		derivative := parse value: input get value: derivative.
		output write: derivative])
			contentsSpecies: Array;
			yourself.! !

!XTPositionTrackingStream methodsFor: 'accessing' stamp: 'fbs 2/11/2013 22:21'!
read: anInteger into: aSequenceableCollection at: startIndex
	| read lineCounts |
	read := source read: anInteger into: aSequenceableCollection at: startIndex.
	columnNumber := columnNumber + read.
	lineCounts := (aSequenceableCollection select: [:o | o == Character lf]) size.
	lineCounts > 0 ifTrue: [
		lineNumber := lineNumber + lineCounts.
		columnNumber := 0].
	lineNumber == 0 ifTrue: [lineNumber := 1].
	^ read.! !

!XTReadStream methodsFor: '*Parsing-Derivatives' stamp: 'fbs 10/31/2012 20:48'!
interleave: aXTReadStream
	"Interleave my values with those of aXTReadStream. If either of us are exhausted, read from the other. If we are both exhausted, signal so with an Incomplete."
	| readFromLeft |
	readFromLeft := true.
	^ self transforming: [:input :output | | next |
		next := [| s |
			s := readFromLeft ifTrue: [input] ifFalse: [aXTReadStream].
			readFromLeft := readFromLeft not.
			s].
		[output put: next value get]
			on: Incomplete do: [output put: next value get]].! !

!XTReadStream methodsFor: '*Parsing-Derivatives' stamp: 'fbs 4/15/2013 20:57'!
parsing: aParser
	^ self positionTracking parsing: aParser.! !

!XTReadStream methodsFor: '*Parsing-Derivatives' stamp: 'fbs 2/11/2013 21:04'!
positionTracking
	"Return an XTReadStream capable of calculating the current line/column number."
	^ XTPositionTrackingStream on: self.! !

!Symbol methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/18/2012 18:55'!
arity
	"A Symbol without a : takes one argument - the receiver."
	^ self numArgs + 1.! !

!Symbol methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/20/2012 22:07'!
lfpWithBottom: anObject
	(self arity = 1) ifFalse: [(ArityMismatch actual: self arity expected: 1) signal].
	^ [:x | self value: x] lfpWithBottom: anObject.! !

!BlockClosure methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/18/2012 18:55'!
arity
	^ self numArgs.! !

!BlockClosure methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 8/30/2012 21:12'!
asParser
	^ DelayedParser on: self.! !

!BlockClosure methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/20/2012 22:04'!
lfpWithBottom: anObject
	"Turn self (a unary BlockClosure) into a recursive (yet monotonic) function over a mutually recursive graph by computing its (least) fixed point."
	self numArgs = 1 ifFalse: [(ArityMismatch actual: self numArgs expected: 1) signal].
	^ LeastFixedPoint run: self bottom: anObject! !

!BlockClosure methodsFor: '*Parsing-Derivatives-Support' stamp: 'fbs 7/17/2012 18:45'!
memoized
	^ MemoizingBlock run: self.! !

Inspector subclass: #ParserInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!ParserInspector commentStamp: 'fbs 10/9/2012 09:26' prior: 0!
To a normal Inspector I add the graphical rendering of that graph.!

!ParserInspector methodsFor: 'accessing' stamp: 'fbs 10/9/2012 09:35'!
fieldList
	^ self baseFieldList , {'<graph>'}! !

!ParserInspector methodsFor: 'private' stamp: 'fbs 10/10/2012 09:22'!
renderGraph: aParser
	^ [aParser printAsDigraph]
		on: Error do: ['Something went wrong rendering this graph. Evaluate ''self printAsDigraph'' to debug it.'].! !

!ParserInspector methodsFor: 'selecting' stamp: 'fbs 10/9/2012 09:37'!
selection
	^ selectionIndex < self fieldList size
		ifTrue: [super selection]
		ifFalse: [self renderGraph: object]! !

!Collection methodsFor: '*Parsing-Derivatives' stamp: 'fbs 10/19/2012 09:47'!
cartesianProduct: aBinaryBlock with: aCollection
	"Given a collection A of a's and B of b's return a collection of all pairs of the form (a,b) where aBinaryBlock is responsible for creating the pairs."
	"self ifEmpty: [^ aCollection]."
	"aCollection ifEmpty: [^ self]."
	^ (self gather: [:a |
		aCollection collect: [:b |
			aBinaryBlock value: a value: b]]) as: self species! !

Object subclass: #DerivingParser
	instanceVariableNames: ''
	classVariableNames: 'DeriverBlock EmptyBlock EmptyParser EmptyStringParser NullableBlock ParseNullBlock'
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!DerivingParser commentStamp: 'fbs 2/21/2013 08:40' prior: 0!
My subclasses implement parsers that represent the _derivative_ of a context free language. Each time you "derive" a new parser with respect to a character you get a parser that
* builds a partial parse tree of the input thus far (the sequence of characters used to derive the current parser), and
* is a parser for the remaining input.

(Of course, you may parse a stream of arbitrary objects, not just characters.)

As a result, my subclasses represent non-blocking parsers: they may consume partial input (say from a socket), storing partial parses until such time as the input is drained.

To put it another way, derivation calculates the parser's continuation.!

DerivingParser subclass: #Cat
	instanceVariableNames: 'first second'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Cat commentStamp: 'fbs 10/16/2012 09:32' prior: 0!
I represent a parser combines two parsers: #first consumes the first part of some stream and, once it has accepted that prefix, #second consumes the rest of the stream.

For example, $a asParser then: $b asParser (only) accepts the input 'ab'.!

!Cat class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 16:35'!
with: firstParser with: secondParser
	^ self new first: firstParser second: secondParser.! !

!Cat methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:03'!
first
	first isDelayed ifTrue: [first := first force].
	^ first.! !

!Cat methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 16:35'!
first: firstParser second: secondParser
	first := firstParser.
	second := secondParser.! !

!Cat methodsFor: 'generating' stamp: 'fbs 10/31/2012 08:09'!
generate
	"Generate the Cartesian product of the two subparsers' output."
	^ ((DelayedStream on: [first generate]) collecting: [:head |
		(DelayedStream on: [second generate]) collecting: [:tail | {head. tail}]]) stitching.! !

!Cat methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:36'!
isCat
	^ true.! !

!Cat methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:06'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3} [label="first"]', Character cr,
	'{2} -> {4} [label="second"]', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self first) printString.
		(parserIntegerMap at: self second) printString.}! !

!Cat methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:03'!
second
	second isDelayed ifTrue: [second := second force].
	^ second.! !

!Cat methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:51'!
subParsers
	^ {self first. self second}.! !

DerivingParser subclass: #DelayedParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!DelayedParser commentStamp: 'fbs 9/6/2012 09:13' prior: 0!
I prevent a parser from being processed prematurely. I act much like a BlockClosure, in that I delay the evaluation of something until asked (via #force).

Parsers that use me often replace me with my forced parser, so my presence in a structure is usually temporary. That makes me look a bit like a Promise.!

!DelayedParser class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 21:14'!
on: aBlock
	^ self new block: aBlock! !

!DelayedParser methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 21:15'!
block: aBlock
	parser := aBlock.! !

!DelayedParser methodsFor: 'delaying' stamp: 'fbs 9/5/2012 09:40'!
delay
	^ self.! !

!DelayedParser methodsFor: 'printing' stamp: 'fbs 9/6/2012 09:41'!
dotLabel
	^ 'Delay'! !

!DelayedParser methodsFor: 'delaying' stamp: 'fbs 2/1/2013 08:49'!
force
	parser isBlock ifTrue: [parser := parser value].
	^ parser.! !

!DelayedParser methodsFor: 'generating' stamp: 'fbs 10/25/2012 18:56'!
generate
	^ self force generate.! !

!DelayedParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 21:15'!
isDelayed
	^ true.! !

!DelayedParser methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:09'!
parser
	parser isBlock ifTrue: [parser := parser value].
	^ parser.! !

!DelayedParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:08'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3}', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self parser) printString.
	}.! !

!DelayedParser methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:41'!
subParsers
	^ {self force}.! !

DerivingParser subclass: #DelegateParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!DelegateParser commentStamp: 'fbs 8/31/2012 12:08' prior: 0!
I represent a reference to another parser. Use me to construct cycles in the parser structure:

	s := DelegateParser new.
	aStar := ($a asParser then: s) or: EmptyString new.
	s parser: aStar.!

!DelegateParser methodsFor: 'printing' stamp: 'fbs 9/6/2012 09:23'!
dotLabel
	^ 'Delegate'! !

!DelegateParser methodsFor: 'generating' stamp: 'fbs 10/26/2012 09:30'!
generate
	^ parser generate.! !

!DelegateParser methodsFor: 'testing' stamp: 'fbs 9/11/2012 19:56'!
isDelegate
	^ true.! !

!DelegateParser methodsFor: 'testing' stamp: 'fbs 9/12/2012 09:33'!
isResolved
	^ parser notNil.! !

!DelegateParser methodsFor: 'accessing' stamp: 'fbs 10/10/2012 19:07'!
parser
	parser ifNil: [UnresolvedDelegate accessing: self].
	^ parser.! !

!DelegateParser methodsFor: 'accessing' stamp: 'fbs 8/31/2012 12:10'!
parser: aParser
	parser := aParser.! !

!DelegateParser methodsFor: 'printing' stamp: 'fbs 10/15/2012 19:43'!
printAsDotNode: parserIntegerMap
	| result |
	result := '{1}' format: {super printAsDotNode: parserIntegerMap}.
	self parser ifNotNil: [
		result := result , ('{1} -> {2}', Character cr) format: {
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self parser) printString}].
	^ result.! !

!DelegateParser methodsFor: 'accessing' stamp: 'fbs 9/12/2012 09:28'!
subParsers
	^ parser
		ifNil: [#()]
		ifNotNil: [{parser}].! !

!DelegateParser methodsFor: 'evaluating' stamp: 'fbs 9/12/2012 18:57'!
whenNullable: aUnaryBlock
	"Usually a DelegateParser will have a subparser. During compaction, the delegates in the compacted form might not be resolved because compaction hasn't completed. In that case, we treat the delegate as not being nullable (i.e., its parseNull collection isn't a single element)."
	self isResolved ifTrue:
		[super whenNullable: aUnaryBlock].! !

!DerivingParser class methodsFor: 'private' stamp: 'fbs 2/12/2013 17:23'!
deriverBlock
	| d |
	d := nil.
	d := [:o :p |
		p class caseOf: {
			[Empty] -> [p].
			[EmptyString] -> [DerivingParser emptyExpected: #Nothing actual: o].
			[EpsStar] -> [DerivingParser emptyExpected: #EndOfStream actual: o].
			[Literal] -> [(o = p token)
							ifTrue: [EpsStar treeSet: {o}]
							ifFalse: [DerivingParser emptyExpected: p token actual: o]].
			[LiteralSet] -> [(p tokens includes: o)
				ifTrue: [EpsStar treeSet: {o}]
				ifFalse: [DerivingParser emptyExpected: p tokens actual: o]].
			[StarParser] -> [(d value: o value: p parser) then: p].
			[Union] -> [[d value: o value: p left] asParser
				or: [d value: o value: p right] asParser].
			[Cat] -> [p first isNullable
				ifTrue: [[[EpsStar treeSet: p first parseNull] asParser
							then: [d value: o value: p second] asParser] asParser
						or: [[d value: o value: p first] asParser
							then: p second] asParser]
				ifFalse: [[(d value: o value: p first)] asParser then: p second]].
			[Red] -> [(d value: o value: p parser) reduce: p reducer].
			[DelayedParser] -> [d value: o value: p force].
			[DelegateParser] -> [DelegateParser new parser: (d value: o value: p parser)]}
		otherwise: [Error signal: 'derivative not defined for ', p className]] memoized.
	^ d! !

!DerivingParser class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 16:00'!
empty
	^ EmptyParser! !

!DerivingParser class methodsFor: 'instance creation' stamp: 'fbs 2/14/2013 20:51'!
emptyExpected: expectedObject actual: actualObject
	^ Empty expected: expectedObject actual: actualObject.! !

!DerivingParser class methodsFor: 'instance creation' stamp: 'fbs 9/4/2012 08:56'!
emptyString
	^ EmptyStringParser! !

!DerivingParser class methodsFor: 'private' stamp: 'fbs 4/13/2013 23:46'!
flushCaches
	EmptyBlock := self isEmptyBlock.
	NullableBlock := self isNullableBlock.
	ParseNullBlock := self parseNullBlock.! !

!DerivingParser class methodsFor: 'class initialization' stamp: 'fbs 2/12/2013 16:22'!
initialize
	EmptyParser := Empty new.
	EmptyStringParser := EmptyString new.
	self flushCaches.! !

!DerivingParser class methodsFor: 'private' stamp: 'fbs 2/8/2013 08:39'!
isEmptyBlock
	| e |
	e := nil.
	e := [:p |
		p class caseOf: {
			[Empty] -> [true].
			[EmptyString] -> [false].
			[EpsStar] -> [false].
			[Literal] -> [false].
			[LiteralSet] -> [false].
			[StarParser] -> [false].
			[Union] -> [(e value: p left) and: [e value: p right]].
			[Cat] -> [(e value: p first) or: [e value: p second]].
			[Red] -> [e value: p parser].
			[DelayedParser] -> [e value: p force].
			[DelegateParser] -> [p isResolved ifTrue: [e value: p parser] ifFalse: ["We can't say!!" false]]}
			otherwise: [Error signal: 'isEmpty not defined for ', p className]] lfpWithBottom: false.
	^ e.! !

!DerivingParser class methodsFor: 'private' stamp: 'fbs 2/8/2013 08:39'!
isNullableBlock
	| n |
	n := nil.
	n := [:p |
		p class caseOf: {
			[Empty] -> [false].
			[EmptyString] -> [true].
			[EpsStar] -> [true].
			[Literal] -> [false].
			[LiteralSet] -> [false].
			[Union] -> [(n value: p left) or: [n value: p right]].
			[Cat] -> [(n value: p first) and: [n value: p second]].
			[Red] -> [n value: p parser].
			[StarParser] -> [(n value: p parser) or: [p parser isEmpty]].
			[DelayedParser] -> [n value: p force].
			[DelegateParser] -> [n value: p parser]}
		otherwise: [Error signal: 'isNullable not defined for ', p className]] lfpWithBottom: false.
	^ n.! !

!DerivingParser class methodsFor: 'private' stamp: 'fbs 2/8/2013 08:39'!
parseNullBlock
	| pn |
	pn := nil.
	pn := [:p |
		p class caseOf: {
			[Empty] -> [Set new].
			[EmptyString] -> [Set newFrom: {#()}].
			[EpsStar] -> [p treeSet].
			[Literal] -> [Set new].
			[LiteralSet] -> [Set new].
			[Union] -> [(pn value: p left) union: (pn value: p right)].
			[Cat] -> [ | concat |
				concat := [:t1 :t2 | t1 asArray , t2 asArray].
				(pn value: p first)
					cartesianProduct: concat
					with: (pn value: p second)].
			[Red] -> [(pn value: p parser) collect: p reducer].
			[StarParser] -> [Set newFrom: #(#())].
			[DelayedParser] -> [pn value: p force].
			[DelegateParser] -> [p isResolved ifFalse: [UnresolvedDelegate accessing: p]. pn value: p parser]}
		otherwise: [Error signal: 'parseNull not defined for ', p className]] lfpWithBottom: Set new.
	^ pn.! !

!DerivingParser methodsFor: 'delaying' stamp: 'fbs 8/30/2012 21:29'!
delay
	^ [self] asParser.! !

!DerivingParser methodsFor: 'printing' stamp: 'fbs 9/6/2012 09:16'!
dotLabel
	^ self className! !

!DerivingParser methodsFor: 'delaying' stamp: 'fbs 9/5/2012 09:49'!
force
	^ self.! !

!DerivingParser methodsFor: 'generating' stamp: 'fbs 10/25/2012 18:42'!
generate
	"Many parsers don't parse anything."
	^ #() reading.! !

!DerivingParser methodsFor: 'inspecting' stamp: 'fbs 10/9/2012 09:33'!
inspectorClass
	^ ParserInspector! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:35'!
isCat
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 10/11/2012 19:12'!
isChildOf: aParser
	aParser subParsers do:
		[:child | PostOrderParserWalker
		walk: child
			doing: [:p | p = self ifTrue: [^ true]]].
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 21:14'!
isDelayed
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/11/2012 19:55'!
isDelegate
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/21/2012 09:48'!
isEmpty
	"Is the language I define I the empty set?"
	^ EmptyBlock value: self.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 2/15/2013 09:29'!
isEmptyParser
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 15:59'!
isEps
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/4/2012 09:04'!
isEpsStar
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:28'!
isLazy
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 15:56'!
isLiteral
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/21/2012 18:47'!
isNullable
	"Can I parse the empty string?"
	^ NullableBlock value: self.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:43'!
isRed
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 9/13/2012 19:35'!
isStar
	^ false.! !

!DerivingParser methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:48'!
isUnion
	^ false.! !

!DerivingParser methodsFor: 'as yet unclassified' stamp: 'fbs 4/17/2013 17:35'!
optional
	^ self or: DerivingParser emptyString! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 2/15/2013 14:49'!
or: aParser
	aParser isEmptyParser ifTrue: [^ self].
	^ Union left: self delay right: aParser delay.! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 9/21/2012 18:50'!
parseNull
	"Can I parse the empty string?"
	^ ParseNullBlock value: self.! !

!DerivingParser methodsFor: 'printing' stamp: 'fbs 9/21/2012 09:27'!
printAsDigraph
	| c intmap i |
	"Post-order makes for nicer graphs."
	i := 0.
	intmap := Dictionary new.
	PreOrderParserWalker walk: self collecting: [:p | intmap at: p put: i. i := i + 1].
	c := (PostOrderParserWalker walk: self collecting: [:p | p printAsDotNode: intmap]) asOrderedCollection.
	c reverseInPlace.
	c addAllFirst: {'digraph {', Character cr. 'ordering=out', Character cr}.
	
	c addLast: ('}', Character cr).
	^ c reduce: #,.! !

!DerivingParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:09'!
printAsDotNode: parserIntegerMap
	"parserIntegerMap maps parsers to integers. We do this manually because #identityHash is not a reliable way (by design) of distinguishing between two objects."
	^ self printNode: parserIntegerMap.! !

!DerivingParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:20'!
printNode: parserIntegerMap
	^ ('{1} [label="{2} ({1})"]' format: {(parserIntegerMap at: self) printString. self dotLabel}),
		Character cr.! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 10/25/2012 09:22'!
reduce: aUnaryBlock
	^ Red parser: self reduceWith: aUnaryBlock! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 9/13/2012 19:36'!
star
	^ StarParser repeating: self.! !

!DerivingParser methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:34'!
subParsers
	"Most parsers don't have subparsers."
	^ #().! !

!DerivingParser methodsFor: 'parsing' stamp: 'fbs 2/18/2013 08:30'!
then: aParser
	aParser isEmptyParser ifTrue: [^ aParser].
	^ Cat with: self delay with: aParser delay.! !

!DerivingParser methodsFor: 'evaluating' stamp: 'fbs 9/20/2012 17:13'!
whenNullable: aUnaryBlock
	"If this parser's parse forest contains exactly one tree, pass in that tree to aUnaryBlock."
	| trees |
	trees := self parseNull.
	(trees size = 1) ifTrue: [aUnaryBlock value: trees anyOne].! !

DerivingParser subclass: #Empty
	instanceVariableNames: 'expected actual'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Empty commentStamp: 'fbs 8/30/2012 16:09' prior: 0!
I represent the empty parser.!

!Empty class methodsFor: 'instance creation' stamp: 'fbs 2/14/2013 20:46'!
expected: expectedObject actual: actualObject
	^ self basicNew
		initializeWithExpected: expectedObject
		actual: actualObject.! !

!Empty methodsFor: 'accessing' stamp: 'fbs 2/14/2013 19:09'!
actual
	^ actual.! !

!Empty methodsFor: 'accessing' stamp: 'fbs 2/14/2013 20:49'!
error
	^ (expected isCollection and: [expected isString not])
		ifTrue: [| collDesc |
			collDesc := expected
				ifEmpty: ['']
				ifNotEmpty: [(expected collect: [:o | o printString]) reduce: [:acc :each | acc, ', ', each]].
			'Expected one of <{1}>, found a <{2}>' format: {collDesc. actual printString}]
		ifFalse: ['Expected a <{1}>, found a <{2}>' format: {expected printString. actual printString}].! !

!Empty methodsFor: 'accessing' stamp: 'fbs 2/14/2013 19:09'!
expected
	^ expected.! !

!Empty methodsFor: 'initialize-release' stamp: 'fbs 2/14/2013 20:47'!
initialize
	super initialize.
	expected := nil.
	actual := nil.! !

!Empty methodsFor: 'private' stamp: 'fbs 2/14/2013 20:39'!
initializeWithExpected: expectedObject actual: actualObject
	self initialize.
	expected := expectedObject.
	actual := actualObject.! !

!Empty methodsFor: 'testing' stamp: 'fbs 2/15/2013 09:31'!
isEmptyParser
	^ true.! !

!Empty methodsFor: 'parsing' stamp: 'fbs 8/30/2012 18:17'!
or: aParser
	^ aParser.! !

!Empty methodsFor: 'parsing' stamp: 'fbs 2/15/2013 14:36'!
reduce: aBlock
	^ self.! !

!Empty methodsFor: 'parsing' stamp: 'fbs 8/30/2012 18:03'!
then: aParser
	^ self.! !

DerivingParser subclass: #EmptyString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!EmptyString commentStamp: 'fbs 8/30/2012 16:23' prior: 0!
I represent a parser that may parse the empty string.!

!EmptyString methodsFor: 'converting' stamp: 'fbs 10/23/2012 18:55'!
asCollection
	^ #().! !

!EmptyString methodsFor: 'printing' stamp: 'fbs 10/15/2012 19:59'!
dotLabel
	^ 'eps'.! !

!EmptyString methodsFor: 'generating' stamp: 'fbs 10/29/2012 12:03'!
generate
	^ #(()) reading! !

!EmptyString methodsFor: 'testing' stamp: 'fbs 9/4/2012 09:07'!
isEps
	^ true.! !

!EmptyString methodsFor: 'printing' stamp: 'fbs 9/18/2012 16:00'!
printAsDotNode: parserIntegerMap
	^ ('{1} [label="{2} ({1})"]' format: {
		(parserIntegerMap at: self) printString.
		self dotLabel}),
	Character cr.! !

EmptyString subclass: #EpsStar
	instanceVariableNames: 'treeSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!EpsStar commentStamp: 'fbs 8/30/2012 16:10' prior: 0!
I represent an intermediate state in the parsing of some input. treeSet forms the collective seed of a set of parse trees.!

!EpsStar class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 15:58'!
treeSet: aCollection
	^ self new treeSet: aCollection asSet.! !

!EpsStar methodsFor: 'printing' stamp: 'fbs 10/24/2012 09:39'!
dotLabel
	^ 'eps*'.! !

!EpsStar methodsFor: 'generating' stamp: 'fbs 10/29/2012 12:05'!
generate
	"An eps* node represents a partial parse: it's an internal construct and should never appear in an initial grammar."
	^ #() reading.! !

!EpsStar methodsFor: 'testing' stamp: 'fbs 9/4/2012 09:05'!
isEps
	"This is rubbish. We should actually have the eps and eps* be subclasses of a common ancestor."
	self flag: #todo.
	^ false.! !

!EpsStar methodsFor: 'testing' stamp: 'fbs 9/4/2012 09:04'!
isEpsStar
	^ true.! !

!EpsStar methodsFor: 'printing' stamp: 'fbs 10/24/2012 09:38'!
printAsDotNode: parserIntegerMap
	^ ('{1} [label="{2} ({3}) ({4})"]' format: {
		(parserIntegerMap at: self) printString.
		self dotLabel.
		self treeSet printString.
		(parserIntegerMap at: self) printString}),
	Character cr.! !

!EpsStar methodsFor: 'accessing' stamp: 'fbs 8/30/2012 15:59'!
treeSet
	^ treeSet! !

!EpsStar methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 15:58'!
treeSet: aCollection
	treeSet := aCollection.! !

DerivingParser subclass: #Literal
	instanceVariableNames: 'token'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Literal commentStamp: 'fbs 8/30/2012 16:10' prior: 0!
I represent a parser that accepts some literal. Note that parsing with derivatives is not limited to parsing streams of characters, so my literal could be any object (not already parsed by other parsers).!

!Literal class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 15:55'!
token: anObject
	^ self new token: anObject! !

!Literal methodsFor: 'generating' stamp: 'fbs 10/26/2012 18:54'!
generate
	^ {token} reading.! !

!Literal methodsFor: 'testing' stamp: 'fbs 8/30/2012 15:56'!
isLiteral
	^ true.! !

!Literal methodsFor: 'printing' stamp: 'fbs 9/17/2012 18:54'!
printAsDotNode: parserIntegerMap
	^ ('{1} [label="Literal ({2}) ({1})"]' format: {
		(parserIntegerMap at: self) printString.
		self token printString}),
	Character cr.! !

!Literal methodsFor: 'accessing' stamp: 'fbs 9/5/2012 10:00'!
token
	^ token.! !

!Literal methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 15:55'!
token: anObject
	token := anObject! !

DerivingParser subclass: #LiteralSet
	instanceVariableNames: 'token tokens'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!LiteralSet class methodsFor: 'as yet unclassified' stamp: 'fbs 1/18/2013 08:49'!
tokens: someObjects
	^ self new tokens: someObjects! !

!LiteralSet methodsFor: 'as yet unclassified' stamp: 'fbs 2/1/2013 17:29'!
generate
	^ tokens reading.! !

!LiteralSet methodsFor: 'as yet unclassified' stamp: 'fbs 1/18/2013 08:47'!
isLiteral
	^ true.! !

!LiteralSet methodsFor: 'as yet unclassified' stamp: 'fbs 1/18/2013 08:48'!
printAsDotNode: parserIntegerMap
	^ ('{1} [label="LiteralSet ({2}) ({1})"]' format: {
		(parserIntegerMap at: self) printString.
		tokens printString}),
	Character cr.! !

!LiteralSet methodsFor: 'as yet unclassified' stamp: 'fbs 1/18/2013 08:48'!
tokens
	^ tokens.! !

!LiteralSet methodsFor: 'as yet unclassified' stamp: 'fbs 1/18/2013 08:48'!
tokens: someObjects
	tokens := someObjects.! !

DerivingParser subclass: #Red
	instanceVariableNames: 'parser reducer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Red commentStamp: 'fbs 8/30/2012 20:46' prior: 0!
I represent a "reducing" parser: a parser that applies a certain function to every element of my subparser's partial parse trees.

I am the parser that allows you to express your semantic actions (such as building ASTs).!

!Red class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 16:45'!
parser: aParser reduceWith: aUnaryBlock
	^ self new parser: aParser reduceWith: aUnaryBlock.! !

!Red methodsFor: 'printing' stamp: 'fbs 10/10/2012 09:45'!
dotLabel
	^ '{1} {2}' format: {self className. [self reducer decompile printString] on: Error do: ['??']}! !

!Red methodsFor: 'generating' stamp: 'fbs 10/25/2012 18:50'!
generate
	^ parser generate collecting: reducer.! !

!Red methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:45'!
isRed
	^ true.! !

!Red methodsFor: 'accessing' stamp: 'fbs 8/30/2012 22:09'!
parser
	^ parser.! !

!Red methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 16:45'!
parser: aParser reduceWith: aUnaryBlock
	parser := aParser.
	reducer := aUnaryBlock.! !

!Red methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:14'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3}', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self parser) printString.
	}.! !

!Red methodsFor: 'parsing' stamp: 'fbs 10/25/2012 09:22'!
reduce: aUnaryBlock
	^ Red parser: self parser reduceWith: (aUnaryBlock compose: self reducer)! !

!Red methodsFor: 'accessing' stamp: 'fbs 8/30/2012 20:40'!
reducer
	^ reducer.! !

!Red methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:35'!
subParsers
	^ {parser}.! !

DerivingParser subclass: #StarParser
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!StarParser commentStamp: 'fbs 10/16/2012 09:34' prior: 0!
I represent the repetition of some language.

For instance, ($a asParser then: $b asParser) star accepts 'ab', 'abab', 'ababab', ...!

!StarParser class methodsFor: 'instance creation' stamp: 'fbs 9/13/2012 19:36'!
repeating: aParser
	^ self new parser: aParser.! !

!StarParser methodsFor: 'generating' stamp: 'fbs 10/29/2012 12:21'!
generate
	"Generate ever-increasing parse trees from the underlying parser p: first 'no occurrences of p', then p, then pp, and so on."
	| count |
	count := 1.
	^ DerivingParser emptyString generate ,
		[ | s |
		s := ((1 to: count) collect: [:ignore | parser generate]) reading stitching.
		count := count + 1.
		"Return an array of parse trees from the underlying parser."
		{s rest} reading] reading stitching.! !

!StarParser methodsFor: 'testing' stamp: 'fbs 9/13/2012 19:37'!
isStar
	^ true.! !

!StarParser methodsFor: 'accessing' stamp: 'fbs 9/13/2012 19:49'!
parser
	^ parser.! !

!StarParser methodsFor: 'initialize-release' stamp: 'fbs 9/13/2012 19:37'!
parser: aParser
	parser := aParser.! !

!StarParser methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:09'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3}', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self parser) printString.
	}.! !

!StarParser methodsFor: 'accessing' stamp: 'fbs 9/14/2012 20:48'!
subParsers
	^ {parser}.! !

DerivingParser subclass: #Union
	instanceVariableNames: 'left right'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives'!

!Union commentStamp: 'fbs 10/16/2012 09:33' prior: 0!
I represent a parser combines two parsers: both parsers consume some stream.

For example, $a asParser or: $b asParser (only) accepts input from the #('a' 'b') asSet.!

!Union class methodsFor: 'instance creation' stamp: 'fbs 8/30/2012 16:50'!
left: lParser right: rParser
	^ self new left: lParser right: rParser.! !

!Union methodsFor: 'generating' stamp: 'fbs 10/31/2012 14:54'!
generate
	^ (DelayedStream on: [left generate])
		interleave: (DelayedStream on: [right generate]).! !

!Union methodsFor: 'testing' stamp: 'fbs 8/30/2012 16:52'!
isUnion
	^ true.! !

!Union methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:47'!
left
	left isDelayed ifTrue: [left := left force].
	^ left.! !

!Union methodsFor: 'initialize-release' stamp: 'fbs 8/30/2012 16:51'!
left: lParser right: rParser
	left := lParser.
	right := rParser.! !

!Union methodsFor: 'printing' stamp: 'fbs 9/14/2012 21:06'!
printAsDotNode: parserIntegerMap
	^ ('{1}',
	'{2} -> {3} [label="left"]', Character cr,
	'{2} -> {4} [label="right"]', Character cr) format: {
		super printAsDotNode: parserIntegerMap.
		(parserIntegerMap at: self) printString.
		(parserIntegerMap at: self left) printString.
		(parserIntegerMap at: self right) printString.}! !

!Union methodsFor: 'accessing' stamp: 'fbs 8/31/2012 08:47'!
right
	right isDelayed ifTrue: [right := right force].
	^ right.! !

!Union methodsFor: 'accessing' stamp: 'fbs 9/5/2012 19:51'!
subParsers
	^ {self left. self right.}.! !

Object subclass: #LeastFixedPoint
	instanceVariableNames: 'changed running visited cache bottom block'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!LeastFixedPoint commentStamp: 'fbs 7/13/2012 18:48' prior: 0!
I calculate the least fixed point of some calculation. That is, given a calculation f and an initial value x, I repeatedly calculate x := f value: x until x == f value x.

My implementation comes from a translation of Matt Might's derp library, from his "YACC is Dead" work.!

!LeastFixedPoint class methodsFor: 'instance creation' stamp: 'fbs 7/13/2012 09:45'!
run: aBlock bottom: anObject
	^ self new run: aBlock bottom: anObject.! !

!LeastFixedPoint methodsFor: 'initialize-release' stamp: 'fbs 7/20/2012 19:00'!
run: aBlock bottom: anObject
	self initialize.
	cache := WeakDictionary new.
	changed := DelimitedDynamicVariable default: #errorChanged.
	running := DelimitedDynamicVariable default: false.
	visited := DelimitedDynamicVariable default: #errorVisited.
	block := aBlock.
	bottom := anObject.! !

!LeastFixedPoint methodsFor: 'evaluating' stamp: 'fbs 8/7/2012 07:56'!
value: anObject
	| cached cachedValue run |
	cached := cache includesKey: anObject.
	cachedValue := cache at: anObject ifAbsent: [bottom].
	run := running dref.
	
	(cached and: [run not]) ifTrue: [^ cachedValue].
	(run and: [visited dref includesKey: anObject]) ifTrue:
		[^ cached ifTrue: [cachedValue] ifFalse: [bottom]].
	
	run ifTrue: [ | newVal |
		visited dref at: anObject put: true.
		newVal := block value: anObject.
		newVal ~= cachedValue ifTrue: [
			changed dset: true.
			cache at: anObject put: newVal].
		^ newVal].
	
	"Technically this condition's not necessary. It causes all the cases to be non-overlapping."
	(cached not and: [run not]) ifTrue: [
		changed, running, visited dlet: {true. true. WeakDictionary new} in: [ | v |
			v := anObject.
			[changed dref] whileTrue: [
				changed dset: false.
				visited dset: WeakDictionary new.
				v := self value: anObject].
			^ v]].
	
	Error signal: 'It''s impossible to reach here.'.! !

Object subclass: #MemoizingBlock
	instanceVariableNames: 'block cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!MemoizingBlock commentStamp: 'fbs 7/17/2012 13:50' prior: 0!
A MemoizingBlock is a block-like thing that remembers the results of its previous invocations. On repeated invocations, it returns this stored result rather than re-executing.!

!MemoizingBlock class methodsFor: 'as yet unclassified' stamp: 'fbs 7/17/2012 13:53'!
run: aBlock
	^ self new run: aBlock! !

!MemoizingBlock methodsFor: 'initialize-release' stamp: 'fbs 8/14/2012 22:36'!
initialize
	cache := Dictionary new.! !

!MemoizingBlock methodsFor: 'initialize-release' stamp: 'fbs 8/2/2012 09:26'!
run: aBlock
	block := aBlock.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:15'!
value: anObject
	^ self valueWithArguments: {anObject}.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:15'!
value: anObject value: anotherObject
	^ self valueWithArguments: {anObject. anotherObject}.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:24'!
value: objectA value: objectB value: objectC
	^ self valueWithArguments: {objectA. objectB. objectC}.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:24'!
value: objectA value: objectB value: objectC value: objectD
	^ self valueWithArguments: {objectA. objectB. objectC. objectD}.! !

!MemoizingBlock methodsFor: 'evaluating' stamp: 'fbs 7/17/2012 20:15'!
valueWithArguments: someObjects
	^ cache at: someObjects ifAbsentPut: [block valueWithArguments: someObjects].! !

!Object methodsFor: '*Parsing-Derivatives' stamp: 'fbs 9/14/2012 09:33'!
asArray
	^ {self}.! !

!Object methodsFor: '*Parsing-Derivatives' stamp: 'fbs 8/30/2012 16:07'!
asParser
	^ Literal token: self.! !

Object subclass: #ParserCompacter
	instanceVariableNames: 'cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!ParserCompacter commentStamp: '<historical>' prior: 0!
I compact parsers. At each step as I traverse the parser graph. I map a parser to its compacted form. When I compact a composite parser (Cat, Union), it will have already had its subparsers mapped (and hence compacted).!

!ParserCompacter class methodsFor: 'instance creation' stamp: 'fbs 2/8/2013 08:28'!
value: aParser
	^ self new value: aParser.! !

!ParserCompacter methodsFor: 'accessing' stamp: 'fbs 2/8/2013 08:29'!
cached: aParser
	"If a delegate doesn't form part of a cycle, it does no harm to point to the delegate's reference instead of the delegate. This also makes the parser one node smaller, potentially."
	| p |
	p := cache at: aParser.
	(p notNil and: [p isDelegate and: [(self formsACycle: p) not]]) ifTrue: [
		^ self cached: p parser].
	^ p.! !

!ParserCompacter methodsFor: 'private' stamp: 'fbs 2/15/2013 14:15'!
compact: aParser
	"Pre-order mark the parser as visited (by storing the parser in the cache); post-order compact the parser (so that all its subparsers (except in a cycle) have been compacted)."
	cache at: aParser ifPresent: [:p | ^ p].
	cache at: aParser put: aParser.
	(aParser subParsers reject: [:p | cache includesKey: p]) do: [:p | self compact: p].
	cache at: aParser put: (aParser class
		caseOf: {
			"Optimise case testing by putting the most common cases first"
			[Empty] -> [aParser].
			[EmptyString] -> [aParser].
			[EpsStar] -> [aParser].
			[Literal] -> [aParser].
			[Cat] -> [self compactCat: aParser].
			[Union] -> [self compactUnion: aParser].
			[Red] -> [self compactRed: aParser].
			[DelayedParser] -> [self compactDelayed: aParser].
			[DelegateParser] -> [self compactDelegate: aParser].
			[StarParser] -> [self compactStar: aParser]}
		otherwise: [aParser]).! !

!ParserCompacter methodsFor: 'private' stamp: 'fbs 4/13/2013 23:28'!
compactCat: aCat
	| c1 c2 |
	c1 := self cached: aCat first.
	c2 := self cached: aCat second.
	
	c1 whenNullable: [:singleToken |
		^ c2 reduce: [:w2 | singleToken asArray , w2 asArray]].
	c1 isEmpty ifTrue: [^ DerivingParser emptyExpected: c1 expected actual: c1 actual].
	
	c2 whenNullable: [:singleToken |
		^ c1 reduce: [:w1 | w1 asArray , singleToken asArray]].
	c2 isEmpty ifTrue: [^ DerivingParser emptyExpected: c2 expected actual: c2 actual].

	^ ((c1 == aCat first) and: [c2 == aCat second])
		ifTrue: [aCat]
		ifFalse: [c1 then: c2]! !

!ParserCompacter methodsFor: 'private' stamp: 'fbs 2/8/2013 08:34'!
compactDelayed: aDelayedParser
	| c |
	c := self cached: aDelayedParser force.
	c == aDelayedParser force
		ifTrue: [^ aDelayedParser].
		
	^ c delay.! !

!ParserCompacter methodsFor: 'private' stamp: 'fbs 2/8/2013 08:45'!
compactDelegate: aDelegateParser
	"The compaction of a delegate parser is the compaction of its subparser. A delegate parser might not be resolvable yet, because if the delegate closes a cycle, the parser to which it should point will not have been mapped yet."
	| mappedSubparser |
	"This delegate completes a cycle. Its subparser hasn't been mapped yet. Mark its presence, and bail. The post-walk will resolve things."
	(cache contains: [:p | p = aDelegateParser parser]) ifFalse: [^ DelegateParser new].
	mappedSubparser := self cached: aDelegateParser parser.
	(aDelegateParser parser == mappedSubparser)
		ifTrue: [^ aDelegateParser].
	(cache includesKey: aDelegateParser) ifTrue: [
		^ DelegateParser new parser: mappedSubparser].
	
	^ DelegateParser new.! !

!ParserCompacter methodsFor: 'private' stamp: 'fbs 2/15/2013 14:33'!
compactRed: aRed
	| sub |
	sub := self cached: aRed parser.
	"Red -> Empty => Empty"
	(sub isEmpty)
		ifTrue: [^ sub].
	"Red -> Red => RedRed"
	(sub isRed)
		ifTrue: [^ sub reduce: aRed reducer].
	"Red -> Delegate -> Red => RedRed"
	(sub isDelegate and: [sub isResolved] and: [sub parser isRed])
		ifTrue: [^ sub parser reduce: aRed reducer].
	"Already compact?"
	(sub == aRed parser) ifTrue: [^ aRed].
	^ sub reduce: aRed reducer.! !

!ParserCompacter methodsFor: 'private' stamp: 'fbs 2/15/2013 14:17'!
compactStar: aStar
	| sub |
	sub := self cached: aStar parser.
	sub isEmpty ifTrue: [^ sub].
	^ aStar! !

!ParserCompacter methodsFor: 'private' stamp: 'fbs 2/14/2013 21:48'!
compactUnion: aUnion
	| c1 c2 |
	c1 := self cached: aUnion left.
	c2 := self cached: aUnion right.
	(c1 isEmpty and: [c2 isEmpty]) ifTrue: [
		^ DerivingParser
			emptyExpected: (c1 expected asArray , c2 expected asArray)
			actual: c2 actual].
	c1 isEmpty ifTrue: [^ c2].
	c2 isEmpty ifTrue: [^ c1].

	^ ((c1 == aUnion left) and: [c2 == aUnion right])
		ifTrue: [aUnion]
		ifFalse: [c1 or: c2]! !

!ParserCompacter methodsFor: 'testing' stamp: 'fbs 2/8/2013 08:28'!
formsACycle: aDelegateParser
	"If aDelegateParser is not yet resolved, treat it as though it forms a cycle."
	aDelegateParser isResolved ifFalse: [^ true].
	^ aDelegateParser isChildOf: aDelegateParser parser.! !

!ParserCompacter methodsFor: 'initialize-release' stamp: 'fbs 2/8/2013 08:28'!
initialize
	super initialize.
	cache := Dictionary new.! !

!ParserCompacter methodsFor: 'private' stamp: 'fbs 2/8/2013 17:20'!
resolveDelegates: aParser
	"During compaction we sometimes introduce cycle-closing delegates. After compaction, we need to go back and fix these delegates."
	PreOrderParserWalker
		walk: aParser
		doing: [:p |
			(p isDelegate and: [p isResolved not])
				ifTrue: [ | preimage |
					preimage := cache keyAtValue: p.
					p parser: (self cached: preimage parser)]].! !

!ParserCompacter methodsFor: 'evaluating' stamp: 'fbs 2/8/2013 08:37'!
value: aParser
	| visited |
	cache at: aParser ifPresent: [:compacted | ^ compacted].
	visited := IdentitySet new.
	self compact: aParser.
	self resolveDelegates: (cache at: aParser).
	^ cache at: aParser.! !

Object subclass: #ParserWalker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!ParserWalker commentStamp: 'fbs 10/11/2012 09:26' prior: 0!
I am a cycle-safe walker of a graph structure. I can perform collection-like operations on the nodes of a graph: doing and collecting.

My children implement different ways of walking a graph.!

!ParserWalker class methodsFor: 'enumerating' stamp: 'fbs 9/12/2012 09:31'!
walk: aParser collecting: aUnaryBlock
	"Walk all parsers reachable from aParser, executing aUnaryBlock on each parser and returning the collection of results (in traversal order). By default ParserWalker will use a cycle-safe pre-order traversal. Invoke #walk:collecting: on subclasses for other traversals."
	^ ((self == ParserWalker ifTrue: [PreOrderParserWalker] ifFalse: [self])
		walk: aParser
		collectingMap: aUnaryBlock)
			pairsCollect: [:a :b | b].! !

!ParserWalker class methodsFor: 'private' stamp: 'fbs 9/8/2012 21:43'!
walk: aParser collecting: aUnaryBlock withCache: someParsers andMap: someAssocs
	self subclassResponsibility.! !

!ParserWalker class methodsFor: 'enumerating' stamp: 'fbs 9/8/2012 23:15'!
walk: aParser collectingMap: aUnaryBlock
	| cache mappedValues |
	"mappedValues contains a plist: {key1. value1. key2. value2. ...}. We don't use a Dictionary because we might care about the order in which we walk the graph, and hence the order of mapped values. cache contains all visited parsers, to render the traversal cycle-safe."
	cache := IdentitySet new.
	mappedValues := OrderedCollection new.
	^ (self walk: aParser collecting: aUnaryBlock withCache: cache andMap: mappedValues) second.! !

!ParserWalker class methodsFor: 'enumerating' stamp: 'fbs 9/12/2012 09:31'!
walk: aParser doing: aUnaryBlock
	"Walk all parsers reachable from aParser, executing aUnaryBlock on each parser. By default ParserWalker will use a cycle-safe pre-order traversal. Invoke #walk:doing: on subclasses for other traversals."
	((self == ParserWalker ifTrue: [PreOrderParserWalker] ifFalse: [self])
		walk: aParser
		collectingMap: aUnaryBlock)
			pairsCollect: [:a :b | b].
	^ self.! !

ParserWalker subclass: #PostOrderParserWalker
	instanceVariableNames: ''
	classVariableNames: 'NotYetResolvedMarker'
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!PostOrderParserWalker commentStamp: 'fbs 10/16/2012 09:30' prior: 0!
I am a cycle-safe post-order depth first graph traversal.!

!PostOrderParserWalker class methodsFor: 'class initialization' stamp: 'fbs 9/8/2012 12:35'!
initialize
	"My presence in a cache represents that (a) this node has been visited but (b) its mapped value has not been calculated (and will be after tree traversal unwinds)."
	NotYetResolvedMarker := Object new.! !

!PostOrderParserWalker class methodsFor: 'private' stamp: 'fbs 9/8/2012 23:21'!
walk: aParser collecting: aUnaryBlock withCache: someParsers andMap: someAssocs
	| newCache newMap |
	(someParsers includes: aParser) ifTrue: [^ {someParsers. someAssocs}].
	
	newMap := someAssocs copy.
	newCache := someParsers copy.
	newCache add: aParser.
	aParser subParsers collect: [:p | | result |
		result := self walk: p collecting: aUnaryBlock withCache: newCache andMap: newMap.
		newCache addAll: result first.
		result second pairsDo: [:a :b |
			(newMap includes: a) ifFalse: [newMap addAll: {a. b}]]].
	newMap addAll: {aParser. aUnaryBlock value: aParser}.
	^ {newCache. newMap}.! !

ParserWalker subclass: #PreOrderParserWalker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Parsing-Derivatives-Support'!

!PreOrderParserWalker commentStamp: 'fbs 10/16/2012 09:30' prior: 0!
I am a cycle-safe pre-order depth first graph traversal.!

!PreOrderParserWalker class methodsFor: 'private' stamp: 'fbs 9/8/2012 23:18'!
walk: aParser collecting: aUnaryBlock withCache: someParsers andMap: someAssocs
	| newCache newMap |
	(someParsers includes: aParser) ifTrue: [^ {someParsers. someAssocs}].
	
	newMap := someAssocs copy.
	newCache := someParsers copy.
	newCache add: aParser.
	newMap addAll: {aParser. aUnaryBlock value: aParser}.
	aParser subParsers collect: [:p | | result |
		result := self walk: p collecting: aUnaryBlock withCache: newCache andMap: newMap.
		newCache addAll: result first.
		result second pairsDo: [:a :b |
			(newMap includes: a) ifFalse: [newMap addAll: {a. b}]]].
	^ {newCache. newMap}.! !
DerivingParser initialize!
PostOrderParserWalker initialize!
